unit Matrix;

interface
uses
   Windows, Messages, SysUtils, Classes, StdCtrls;

type
Matrix23 = array[1..115,1..115] of Real;
function MatrixAdd(Row,Col:integer;sValue1,sValue2:string):string;
function MatrixMul_C(Row,Col:integer;sValue:string;fValue:real):string;
function MatrixMul(Row1,Col1:integer;DimA:Matrix23;Row2,Col2:integer;DimB:Matrix23):Matrix23;
function MatrixTrans(Row,Col:integer;DimA:Matrix23):Matrix23;
function MatrixInverse(Row,Col:integer;DimA:Matrix23):Matrix23;

implementation


function MatrixAdd(Row,Col:integer;sValue1,sValue2:string):string;
var
    i,j : integer;
    iPos : integer;
    MatrixA,MatrixB : Matrix23;
begin
   for i := 1 to Row do
      for j := 1 to Col do
      begin
         iPos := Pos('\',sValue1);
         MatrixA[i,j] := StrToFloat(Copy(sValue1,1,iPos-1));
         sValue1 := Copy(sValue1,iPos+1,length(sValue1));

         iPos := Pos('\',sValue2);
         MatrixB[i,j] := StrToFloat(Copy(sValue2,1,iPos-1));
         sValue2 := Copy(sValue2,iPos+1,length(sValue2));
      end;

   Result := '';
   for  i := 1 to Row do
      for j := 1 to Col do
          Result := Result + floattostrF(MatrixA[i,j] + MatrixB[i,j],ffGeneral,7,4) + '\';

end;


function MatrixMul_C(Row,Col:integer;sValue:string;fValue:real):string;
var
    i,j : integer;
    iPos : integer;
    MatrixA : Matrix23;
begin
   for i := 1 to Row do
      for j := 1 to Col do
      begin
         iPos := Pos('\',sValue);
         MatrixA[i,j] := StrToFloat(Copy(sValue,1,iPos-1));
         sValue := Copy(sValue,iPos+1,length(sValue));
      end;

   Result := '';
   for  i := 1 to Row do
      for j := 1 to Col do
          Result := Result + floattostrF(fValue * MatrixA[i,j],ffGeneral,7,4) + '\';
end;


function MatrixMul(Row1,Col1:integer;DimA:Matrix23;Row2,Col2:integer;DimB:Matrix23):Matrix23;
var
   i,j,k : integer;
   fValue : real;
begin
   for  i := 1 to Row1 do
      for j := 1 to Col2 do
      begin
         fValue := 0.0;
         for k := 1 to Col1 do
             fValue  := fValue + DimA[i,k]*DimB[k,j];
         Result[i,j] := fValue;
      end;
end;


function MatrixTrans(Row,Col:integer;DimA:Matrix23):Matrix23;
var
   i,j : integer;
begin
   for i := 1 to Row do
      for j := 1 to Col do
         Result[j,i] := DimA[i,j];
end;


function Determinant(Row,Col:integer;sValue:string):real;
var
   M :  Matrix23;
 i,j,k : integer;
 iPos,n : integer;
begin
   Result := 0.0;

   for i := 1 to Row do
      for j := 1 to Col do
      begin
         iPos := Pos('\',sValue);
         M[i,j] := StrToFloat(Copy(sValue,1,iPos-1));
         sValue := Copy(sValue,iPos+1,length(sValue));
      end;

   if Row >  3 then
   begin
      n := 1;
      for i := 1 to Row do
      begin
         sValue := '';
         for j := 2 to Row do
             for k := 1 to col do
                if k <> i then
                   sValue := sValue + Floattostrf(M[j,k],ffGeneral,7,4) + '\';

          Result := Result + n*M[1,i]*Determinant(Row-1,Col-1,sValue);
          n := -1 * n;
      end;
   end
   else
   begin
      case Row of
      1 : Result := M[1,1];
      2 : Result := M[1,1]*M[2,2]-M[1,2]*M[2,1];
      3 : Begin
             Result := M[1,1]*M[2,2]*M[3,3] + M[2,1]*M[3,2]*M[1,3]+M[3,1]*M[1,2]*M[2,3];
             Result := Result -M[3,1]*M[2,2]*M[1,3] - M[2,3]*M[3,2]*M[1,1] - M[2,1]*M[1,2]*M[3,3];
          end;
      end;
   end;
end;


function MatrixInverse(Row,Col:integer;DimA:Matrix23):Matrix23;
var
  i,j,k : integer;
  Temp1,Temp2,RetVal : extended;
  Factor: integer;
//txnfile : textfile;
//r,c : integer;
begin
   RetVal := 1.0;

   for i := 1 to Row do
     for j := 1 to Col do
        if i = j then
          Result[i,j] := 1.0
        else
          Result[i,j] := 0.0;

//assignfile(txnfile,'C:\m0.dat');
//rewrite(txnfile);
//for r := 1 to row do
//begin
//   for c := 1 to col do
//     write(txnfile,MatrixA[r,c]:10:4, '  ');
//   writeln(txnfile);
//end;
//closefile(txnFile);

   for i := 1 to Row - 1 do
   begin
     j := i;
     while j < Row+1 do
       if DimA[j, i] <> 0.0 then break
       else
          inc(j);

     if j = Row+1 then
       Exit;

     // Change Row Begin
     if i <> j then
        for k := 1 to Col do
        begin
           temp1 := DimA[i,k];
           temp2 := Result[i,k];
           DimA[i,k] := DimA[j,k];
           Result[i,k] := Result[j,k];
           DimA[j,k] := Temp1;
           Result[j,k] := Temp2;
        end;
     //Change Row End

     if not Odd(Col) then RetVal := -1 * RetVal;

     for j := i + 1 to Row do
     begin
       Temp1 := DimA[j, i]/DimA[i, i];
       for k := 1 to Col do
       begin
         DimA[j,k] := DimA[j,k] + DimA[i,k] * (-Temp1);
         Result[j,k] := Result[j,k] + Result[i,k] * (-Temp1);
       end;
     end;
   end;

//assignfile(txnfile,'C:\m1.dat' );
//rewrite(txnfile);
//for r := 1 to row do
//begin
//   for c := 1 to col do
//      write(txnfile,MatrixA[r,c]:10:4);
//   writeln(txnfile);
//end;
//closefile(txnFile);

   for i := Row downto 1 do
     for j := i - 1 downto 1 do
     begin
       Temp1 := dimA[j,i]/DimA[i,i];
       for k := 1 to Col do
       begin
         DimA[j,k] := DimA[j,k] + DimA[i,k] * (-Temp1);
         Result[j,k] := Result[j,k] + Result[i,k] * (-Temp1);
       end;
     end;

//assignfile(txnfile,'C:\m2.dat');
//rewrite(txnfile);
//for r := 1 to row do
//begin
//  for c := 1 to col do
//     write(txnfile,MatrixA[r,c]:10:4);
//   writeln(txnfile);
//end;
//closefile(txnFile);

   for i := 1 to Row do
   begin
     RetVal := RetVal * DimA[i,i];
     Temp1 := 1/DimA[i,i];
     for k := 1 to Col do
     begin
        DimA[i,k] := DimA[i,k] * Temp1;
        Result[i,k] := Result[i,k] * Temp1;
     end;
   end;

//assignfile(txnfile,'C:\m3.dat');
//rewrite(txnfile);
//for r := 1 to row do
//begin
 //  for c := 1 to col do
  //    write(txnfile,MatrixA[r,c]:10:4);
  // writeln(txnfile);
//end;
//closefile(txnFile);
end;


end.
