//  Santa Fe Institute Stock Market
//  Genetic Programming in Delphi, Tournament Selection

//   Traders Part

unit Ver2;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Matrix, IniFiles, ExtCtrls, ComCtrls, Math, TeEngine, Series,
  TeeProcs, Chart, FileCtrl;

type
  TForm1 = class(TForm)
    StatusBar1 : TStatusBar;
    Chart1 : TChart;
    Series1 : TFastLineSeries;
    Timer : TTimer;
    Panel1 : TPanel;
    edtMsg : TMemo;
    Panel2 : TPanel;
    btnExit : TButton;
    procedure FormActivate(Sender:TObject);
    procedure ShowGraph(x:integer;y:extended);
    procedure Output_msg(sMsg:string);
    procedure TimerTimer(Sender:TObject);
    procedure btnExitClick(Sender:TObject);
    procedure GetEnv(sIniFile:string);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
CONST
   Individual  = 500;
   FileAmount  = 1000;
   XAmount     = 15;
   ExVarAmount = 15;
   MaxSqrt     = 1.090748E+2466;
   WinSize     = 100;
   Element_Amount = 75;

type
   TwoPoints = ^TwoPointsNode;
   TwoPointsNode = record
      iData : byte;
      R     : real;
      Right : TwoPoints;
      Left  : TwoPoints;
   end;

   OnePoint = ^OnePointNode;
   OnePointNode = record
      iData: byte;
      R    : real;
      Next : OnePoint;
   end;
   Str4 = string[4];
   MatrixType = Array[1..WinSize+11] of real;

var
   piFunctionSet : byte;
   piBinarySet   : byte;
   piUnarySet    : byte;
   piTerminalSet : byte;
   piSetNo       : byte;
   piX_No,piY_No,piExVar_No : byte;
   psConst       : string[1];
   FileName      : string;
   FilePath1,FilePath2 : string;
   DataFile,ExDataFile : string;
   Out1Name,Out2Name : string;
   TreeFile      : string;
   X, Y          : Array[1..XAmount] of extended;
   ExVar         : Array[1..ExVarAmount] of real;
   ElementsTable : Array[1..Element_Amount] of Str4;
   Mape          : Array[1..Individual] of real;
   Wk,Wk2,Profits,Vij1: Array[1..Individual] of extended;
   H, Sigma      : Array[1..FileAmount] of real;
   M, W, Vijs    : Array[1..FileAmount] of extended;
   FileIndex     : Array[1..FileAmount] of integer;
   DepthTable,NodeTable : Array[1..Individual] of integer;
   Disk, Data    : TextFile;
   out1, out2    : TextFile;
   BestFile      : TextFile;
   History,MapeFile : TextFile;
   Gen, GNumber  : integer;
   Queue, This   : OnePoint;
   Pstar, Dstar  : extended;
   First, Second : integer;
   DimA,DimB,DimC: MatrixType;
   Root          : TwoPoints;
   BuyNo,SellNo,BeSNo : integer;
   Out_Count     : integer;
   TBid, TOffer  : real;
   AveDepth,AveNode : real;
   TWealth,TWealth1,TWealth2,TWealth3,TWealth4,TWealth5 : extended;
   VarNode,VarDepth,VarHi,VarVijs : real;
   AveHi, AveVijs : extended;
   AveSigma2, VarSigma2 : extended;
   Succeed,SucceedNo : integer;
   VisitGen,VisitGenNo : integer;
   VisitLib,VisitLibNo : integer;
   Martingale,MartingaleNo : integer;
   pfDiskFree  : real;
   pbExit,pbCancel : boolean;
   Backup,fWait,ExData : Textfile;
   sIniFile    : string;
   piDoDim     : byte;
   psWaitFile  : string;
   piBeginGen  : integer;
   Full        : integer;
   Grow        : integer;
   Reproduct   : integer;
   Mutation    : integer;
   MutProb     : real;    // Mutation Probability = X/1000
   DepthLimit  : byte;    // The depth limit of the tree
   LeaveProb   : integer; // Leave are selected according to the probability = X/10
   MaxExp      : real;    // The maximum domain number of exponent: 11360.0
   MutMode     : byte;
   H0          : real;
   M0          : real;
   Lambda      : real;
   Theta1      : real;
   Theta2      : real;
   Theta3      : real;    // 0.0067 (1/150),  0.0133 (1/75)
   Beta        : real;
   Irate       : real;
   G_Gap       : integer;
   LibraryProb : real;    // The probability of library being selected, X %
   TryNo       : byte;


function ApPath:string;
begin
   result := ExtractFilePath(application.ExEName);
end;


function ReadINI(sSecName,sAppName,sIniFile:string):string;
var
    Myini : TIniFile;
begin
   Myini := TIniFile.Create(sInifile);
   Result := Myini.ReadString(sSecName,sAppName,'Not_Found');
   Myini.Free;
   if result = 'Not_Found' then
      result := '';
end;


function Depth2(T:TwoPoints):integer;
var
    l, r : integer;
begin
   result := 0;
   if (T <> nil) then
   begin
      l := Depth2(T^.Left);
      r := Depth2(T^.Right);
      if l >= r then
         result := l + 1
      else
         result := r + 1;
   end
end;   (* end of Depth *)


function Count_Node(T:TwoPoints):integer;
var
    l, r : integer;
begin
   result := 0;
   if (T <> nil) then
   begin
      l := Count_Node(T^.Left);
      r := Count_Node(T^.Right);
      result := l + r + 1;
   end
end;   (* end of Depth *)


procedure OpenFile(Mode:integer;G:integer);
var
   l : byte;
   i : integer;
   Temp : string;
   sFilePath : string;
begin
   if Mode = -2 then
      sFilePath := FilePath2
   else
      sFilePath := FilePath1;

   Temp := inttostr(G);
   l := Length(Temp);
   for i := l + 1 to 4 do
      Temp := '0' + Temp;

   case G div 100 of
   0: FileName := sFilePath +'0\'+ FileName + Temp + '.dat';
   1: FileName := sFilePath +'1\'+ FileName + Temp + '.dat';
   2: FileName := sFilePath +'2\'+ FileName + Temp + '.dat';
   3: FileName := sFilePath +'3\'+ FileName + Temp + '.dat';
   4: FileName := sFilePath +'4\'+ FileName + Temp + '.dat';
   5: FileName := sFilePath +'5\'+ FileName + Temp + '.dat';
   6: FileName := sFilePath +'6\'+ FileName + Temp + '.dat';
   7: FileName := sFilePath +'7\'+ FileName + Temp + '.dat';
   8: FileName := sFilePath +'8\'+ FileName + Temp + '.dat';
9,10: FileName := sFilePath +'9\'+ FileName + Temp + '.dat';
   end;

      assignfile(Disk,FileName);
      case Mode of
      0,-2: Reset (Disk);
      1: Rewrite(Disk);
      2: begin
            assignfile(BestFile,FileName);
            Rewrite(BestFile);
         end;
      end;
end;  (* end of AssignFile *)


procedure PostOrder(var t:TwoPoints);
var
    ch : str4;
begin
   if t <> nil then
   begin
      PostOrder(t^.Left);
      PostOrder(t^.Right);
      ch := trim(ElementsTable[t^.iData]);
      write(Disk,ch);
      if t^.iData = piSetNo then
         write(Disk,'(',t^.R:4:2,' )');
      t := nil;
   end;
end;


procedure FreeTree(var t:TwoPoints);
begin
   try
      if t = nil then
         exit;

      if t.Left <> nil then
         FreeTree(t.Left);

      if t.Right <> nil then
          FreeTree(t.Left);

      Dispose(t);
      t := nil;
   except
      t := nil;
   end;
end;


procedure FreeQueue(var q:OnePoint);
var ip : OnePoint;
begin
  try
     if q <> nil then
        while q <> nil do
        begin
          ip := q;
          q := q^.next;
          dispose(ip);
        end;
      q := nil;
   except
      q := nil;
   end;
end;


procedure StoreTree(var t:TwoPoints;i:integer);
begin
   FileName := 'T';
   OpenFile(1,i);
   PostOrder(t);
   t := nil;
   closefile(Disk);
end;    (* end of StoreTree *)


function ReadSTR4:str4;
var
    str      : str4;
    ch1, ch2 : char;
begin
    Str := '';
    read(Disk,ch1);
    Str := ch1;
    case ch1 of
    'E','L','C','A',
    'e','l','c','a'
                   : begin
                     read(Disk,ch1,ch2);
                     Str := Concat(Str,ch1,ch2);
                     end;
    'x','X','Y','y','z','Z'
                   : begin
                     read(Disk,ch1,ch2);
                     Str := Concat(Str,ch1,ch2);
                     end;
    'S' ,'s'       : begin
                     read(Disk,ch1);
                     case ch1 of
                     'q','Q'
                            : begin
                              Str := Concat(Str,ch1);
                              read(Disk,ch1,ch2);
                              Str := Concat(Str,ch1,ch2);
                              end;
                     'i','I'
                            : begin
                              read(Disk,ch2);
                              Str := Concat(Str,ch1,ch2);
                              end;
                     end;
                 end;
    end;
    Result := Str;
end;


function Obtain(str:str4):byte;
var
   i : byte;
begin
   Str := trim(Str);
   result := 0;
   for i := 1 to piSetNo do
   begin
      if ElementsTable[i] = Str then
      begin
         result := i;
         exit;
      end;
   end;
end;


procedure ReadTree(i:integer);
var
    q   : OnePoint;
    ch  : char;
    str : str4;
begin
   if i = -1 then
   begin
      assignfile(Disk,TreeFile);
      reset(Disk);
   end
   else
   begin
      FileName := 'T';
      OpenFile(0,i);
   end;
   New(Queue);
   str := ReadSTR4;
   Queue^.iData := Obtain(str);
   if Queue^.iData = piSetNo then
      read(Disk,ch,Queue^.R,ch,ch);
   queue^.next := nil;
   this := queue;
   while not eof(Disk) do
   begin
      New(q);
      str := ReadSTR4;
      q^.iData := obtain(str);
      q^.next := nil;
      if q^.iData = piSetNo then
         read(Disk,ch,q^.R,ch,ch);
      this^.next := q;
      this := this^.next;
   end;  (* end of while not eof *)
   this := queue;
   closefile(Disk);
end;    (* end of ReadTree *)


procedure ReadTree2(i:integer);
var
    q   : OnePoint;
    ch  : char;
    str : str4;
begin
   FileName := 'T';
   OpenFile(-2,i);
   New(Queue);
   str := ReadSTR4;
   Queue^.iData := Obtain(str);
   if Queue^.iData = piSetNo then
      read(Disk,ch,Queue^.R,ch,ch);
   queue^.next := nil;
   this := queue;
   while not eof(Disk) do
   begin
      New(q);
      str := ReadSTR4;
      q^.iData := obtain(str);
      q^.next := nil;
      if q^.iData = piSetNo then
         read(Disk,ch,q^.R,ch,ch);
      this^.next := q;
      this := this^.next;
   end;  (* end of while not eof *)
   this := queue;
   closefile(Disk);
end;    (* end of ReadTree *)


function GetOp(var empty:boolean;var Value:real):byte;
begin
   if This <> nil then
   begin
      Result := This^.iData;
      Value := This^.R;
      This  := This^.Next;
   end
   else
       empty := true;
end;


function Compute(op:byte;value1,value2:extended):extended;
var
   sOp : str4;
begin
   sOp := ElementsTable[Op];
   case sOp[1] of
      '+' : Compute := value2 + value1;
      '-' : Compute := value2 - value1;
      '*' : Compute := value2 * value1;
      '%' : if value1 = 0.0 then
               Compute := 1
            else
               Compute := value2 / value1;
      'E' : begin
             if Abs(Value1) > MaxExp then
                Compute := Exp(MaxExp)
             else
                Compute := Exp(value1);
          end;
      'L': if value1 = 0.0 then
              Compute := 0.0
           else
              Compute := Ln(Abs(value1));
      'S' : begin
               case sOp[2] of
               'Q' : Compute := sqrt(abs(value1));
               'I' : Compute := Sin(value1);
               end;
            end;
    'C' : Compute := Cos(value1);
    'A' : Compute := Abs(value1);
   end;
end;  (* end of Compute *)


procedure ResetWk;
var
   i : integer;
begin
   for i := 1 to Individual do
   begin
      Wk2[i] := Wk[i];
      Wk[i] := 0.0;
   end;
end;


procedure ComputeProfit;
var i : integer;
begin
   if Gen = 1 then
   begin
      for i := 1 to Individual do
      begin
         Profits[i] := Irate * M0 + H0 * (Y[1]-X[2]);
         W[Second+i] := Profits[i] + W[First+i];
         Wk[i] := Profits[i] + Wk[i];
      end;
   end
   else
   begin
     for i := 1 to Individual do
     begin
         Profits[i] := Irate*M[Second+i]+H[Second+i]*(Y[1]-X[2]);
         W[Second+i] := Profits[i] + W[First+i];
         Wk[i] := Profits[i] + Wk[i];
      end;
   end;
end;


procedure ComputeHi(i:integer;var Hi:real);
TYPE
     spoint = ^StackNode;
     StackNode = record
        Value : extended;
        next  : spoint;
     end;
var
    op      : byte;
    Value   : real;
    Top     : spoint;
    empty   : boolean;
    j       : integer;
    Vij     : extended;
    fMean,AveSigma : extended;

   procedure FreeStack(var s:spoint);
   var ip : spoint;
   begin
Dispose(s);
exit;
      try
         while s <> nil do
         begin
            ip := s;
            s := s^.next;
            dispose(ip);
         end;
         s := nil;
      except
         s := nil;
      end;
   end;

   procedure push(value:extended);
   var
       p : spoint;
   begin
      New(p);
      p^.value := value;
      p^.next := Top;
      Top := p;
   end;

   function pop:extended;
   var
        p : spoint;
   begin
      if Top <> nil then
      begin
         p := Top;
         pop := Top^.value;
         Top := Top^.next;
         Dispose(p);
      end
      else
      begin
         MessageDlg('Error: Stack underfollow. ',mtWarning,[mbOK],0);
         exit;
      end
   end;

begin
   try
      This := Queue;
      Top := nil;   (* Initial Stack while compute another record of Data *)
      empty := false;
      op := GetOp(empty,Value);

      while not empty do   (* Do while queue isn't empty *)
      begin
         if (op > piFunctionSet) then  // op in TerminalSet
         begin
            if op = piSetNo then // op is Const
               Push(Value)
            else
            begin
               op := op - piFunctionSet;
               if op <= piX_No then   // op in X Var
                  Push(X[op])
               else
               begin
                  op := op - piX_No;
                  if op <= piY_No then   // op in Y Var
                     Push(Y[op])
                  else
                  begin
                     op := op - piY_No;
                     if op <= piExVar_No then   // op in ExVar
                        Push(ExVar[op])
                     else
                     begin
                        MessageDlg('Incorrect operator !',mtWarning,[mbOK],0);
                        exit;
                     end;
                  end;
               end;
            end;
         end  // end of if (op > piFunctionSet)
         else // op In function set
         begin
            if (op <= piBinarySet) then
               push(compute(op,pop,pop))
                                          (* Note: The compute function *)
            else   (* op is Unary *)            (* is second value "op" *)
               push(compute(op,pop,-1));  (* first value *)
         end;
         op := GetOp(empty,Value);
      end;    (* end of while not empty *)

      if Top^.next <> nil then
      begin
         MessageDlg('Error: Data compute not match in T' ,mtWarning,[mbOK],0);
         exit;
      end;

      Vij := Top^.Value;
      Vij1[i] := Vij;

      if Abs(Vij) <= MaxExp then
         Vij := (Exp(Theta2*Vij)-Exp(-Theta2*Vij))/(Exp(Theta2*Vij)+Exp(-Theta2*Vij))
      else
      begin
         if Vij > 0.0 then
            Vij := 1.0
         else
            Vij := -1.0;
      end;
//    Vij := Y[1] * (1 + Theta1 * Vij);  // Vij := (Pt+Dt)*(1+0.5*Vij)
      Vij := (X[1] + 10.0) * (1 + Theta1 * Vij);
      Vijs[Second+i] := Vij;

      if Abs(Vijs[Second+i]-Y[1]) <= 0.00001 then
         Martingale := 1
      else
         Martingale := 0;

      fMean := 0.0; AveSigma := 0.0;
      for j := 102 to 111 do
         fMean := fMean + (DimA[j]);
      fMean := fMean/10;

      for j := 102 to 111 do
         AveSigma := AveSigma + Sqr(DimA[j] - fMean);
      AveSigma := AveSigma/9;

      Sigma[First+i] := AveSigma;
      Sigma[Second+i] := (1-Theta3)*Sigma[First+i]+Theta3*Sqr(Y[1]-Vijs[First+i]);

      Hi := (Vij-X[1]*(1+Irate))/(Lambda*Sigma[Second+i]);
      if Hi <= 0.0 then
         Hi := 0.0
      else
      begin
         if Abs(Hi) <= 0.00001 then
            Hi := 0.0
         else
         begin
            if W[First+i] - X[1]*Hi <= 0.0 then
               Hi := W[First+i]/X[1];
         end;
      end;
      FreeStack(Top);
   except
      FreeStack(Top);
   end;
end;


procedure ComputeVij(i,u:integer;var Vij:extended);
type
     spoint = ^StackNode;
     StackNode = record
        Value : extended;
        next  : spoint;
     end;
var
    op        : byte;
    Value     : real;
    Top       : spoint;
    empty     : boolean;
    Order,j   : integer;
    fMean,AveSigma,Hi : real;

   procedure FreeStack(var s:spoint);
   var ip : spoint;
   begin
Dispose(s);
exit;
      try
         while s <> nil do
         begin
            ip := s;
            s := s^.next;
            dispose(ip);
         end;
         s := nil;
      except
         s := nil;
      end;
   end;

   procedure push(value:extended);
   var
       p : spoint;
   begin
      New(p);
      p^.value := value;
      p^.next := Top;
      Top := p;
   end;

   function pop:extended;
   var
        p : spoint;
   begin
      if Top <> nil then
      begin
         p := Top;
         pop := Top^.value;
         Top := Top^.next;
         Dispose(p);
      end
      else
      begin
         MessageDlg('Error: Stack underfollow. ',mtWarning,[mbOK],0);
         exit;
      end
   end;

begin
   try
      This := Queue;
      Top := nil;   (* Initial Stack while compute another record of Data *)
      empty := false;
      op := GetOp(empty,Value);
      while not empty do   (* Do while queue isn't empty *)
      begin
         if (op > piFunctionSet) then
         begin
            if op = piSetNo then
               Push(Value)
            else
            begin
               op := op - piFunctionSet;
               if op <= piX_No then
                  Push(DimB[u-1-op])
               else
               begin
                  op := op - piX_No;
                  if op <= piY_No then
                     Push(DimC[u-1-op])
                  else
                  begin
                     op := op - piY_No;
                     if op <= piExVar_No then
                        Push(Exvar[op])
                     else
                     begin
                        MessageDlg('Incorrect operator',mtWarning,[mbOK],0);
                        exit;
                     end;
                  end;
               end;
            end;
         end
         else
         begin
            if (op <= piBinarySet) then
               push(compute(op,pop,pop))
                                          (* Note: The compute function *)
            else   (* op is Unary *)            (* is second value "op" *)
               push(compute(op,pop,-1));  (* first value *)
         end;
         op := GetOp(empty,Value);
      end;    (* end of while not empty *)

      if Top^.next <> nil then
      begin
         MessageDlg('Error: Data compute not match in T' ,mtWarning,[mbOK],0);
         exit;
      end;
      Vij := Top^.Value;

      if Abs(Vij) <= MaxExp then
         Vij := (Exp(Theta2*Vij)-Exp(-Theta2*Vij))/(Exp(Theta2*Vij)+Exp(-Theta2*Vij))
      else
      begin
        if Vij > 0.0 then
           Vij := 1.0
        else
           Vij := -1.0;
      end;
      Vij := DimC[u-1] * (1 + Theta1 * Vij);
//    Vij := (DimB[u-1]+10.0) * (1 + Theta1 * Vij);
//    Vij := (DimB[u-1]+10.0+0.95*(DimC[u-1]-DimB[u-1]-10.0)) * (1 + Theta1 * Vij);

      FreeStack(Top);
      except
        FreeStack(Top);
     end;
end;


procedure MakeFullTree(var t:TwoPoints;d,depth:integer);
var
    p        : TwoPoints;
    i,j,sign : byte;
    val      : real;
begin
   if d < depth then         (* Get element from function set *)
   begin
       New(p);
       p^.R := 111.0;
//     for j := 1 to Random(256) do
//         i := Random(j);
       P^.iData := Random(piFunctionSet) + 1;
       MakeFullTree(p^.left,d+1,depth);
       if (p^.iData <= piBinarySet) then        (* This element is binary *)
          MakeFullTree(p^.right,d+1,depth)
       else
          p^.right := nil;                 (* This element is unary *)
   end
   else
      if d = depth then       (* Get element from terminal set *)
      begin
         New(p);
         p^.R := 111.0;
         p^.iData := Random(piTerminalSet) + piFunctionSet + 1;
         if p^.iData = piSetNo then           (* Choice Const R *)
         begin
            val := Random(10)+Random(100)/100;
            sign := Random(2);
            if sign = 1 then
              val := val*(-1);
            p^.R := val;
         end;
         P^.right := nil;
         p^.left := nil;
      end;
   t := p;
end;    (* end of MakeFullTree *)


procedure MakeGrowTree(var t:TwoPoints;d,depth:integer);
var
    p        : TwoPoints;
    i,j,sign : byte;
    val      : real;
begin
   if d <= depth then
   begin
      if d < depth then     (* d < depth, so get element from all element *)
      begin
//       for j := 1 to Random(256) do
//           i := Random(j);
         i := Random(2);
         if i = 0 then
            i := Random(piFunctionSet) + 1
         else
            i := Random(piTerminalSet) + piFunctionSet + 1;
      end
      else                  (* d = depth, so get element from terminal set *)
      begin
//       for j := 1 to Random(256) do
//           i := Random(j);
         i := Random(piTerminalSet) + piFunctionSet + 1;
      end;
      New(p);
      p^.R := 111.0;
      if (i > piSetNo) or (i <= 0) then
      begin
         MessageDlg('Data Generate Error !!  It order is '+inttostr(i),mtWarning,[],0);
         exit;
      end;
      P^.iData := i;
      if (p^.iData <= piFunctionSet) then
      begin
         MakeGrowTree(p^.left,d+1,depth);
         if (p^.iData <= piBinarySet) then        (* This element is binary *)
            MakeGrowTree(p^.right,d+1,depth)
         else
            p^.right := nil;                 (* This element is unary *)
      end
      else                                   (* This element is terminal *)
      begin
         if i = piSetNo then
         begin
            val := Random(10)+Random(100)/100;
            sign := Random(2);
            if sign = 1 then
               val := val*(-1);
            p^.R := val;
         end;
         P^.right := nil;
         p^.left  := nil;
      end;
      t := p;
   end;    (* end of d <= depth *)
end;    (* end of MakeGrowTree *)


procedure Generate(N:integer);
var
    j, depth : integer;
    t        : TwoPoints;
    Hi       : real;
begin    (* Generate *)
   j := (Individual)*N+1;
   for depth := 2 to 6 do
   begin
      n := 1;
      repeat                 (* Generated tree by full method *)
         MakeFullTree(t,1,depth);
         DepthTable[j] := Depth2(t);
         NodeTable[j] := Count_Node(t);
         StoreTree(t,j);
         FreeTree(t);
         ReadTree(j);         (* Read file of tree i and build a queue list *)
         ComputeHi(j,Hi);
         H[Second+j] := Hi;
         n := n + 1;
         j := j + 1;
         FreeQueue(Queue);      (* Return all the nodes of queue of tree i *)
      until n > Full;
      n := 1;
      repeat
         MakeGrowTree(t,1,depth);
         DepthTable[j] := Depth2(t);
         NodeTable[j] := Count_Node(t);
         StoreTree(t,j);
         FreeTree(t);
         ReadTree(j);         (* Read file of tree i and build a queue list *)
         ComputeHi(j,Hi);
         H[Second+j] := Hi;
         n := n + 1;
         j := j + 1;
         FreeQueue(Queue);     (* Return all the nodes of queue of tree i *)
      until n > Grow;
   end;    (* end of for loop depth *)
end;   (* end of generate *)


procedure InterChange(i,j:integer);
var
   t : integer;
begin
   t := FileIndex[i];
   FileIndex[i] := FileIndex[j];
   FileIndex[j] := t;
end;


procedure sort(m,n:integer);
var
   i, j : integer;
   k : extended;
begin
   if m < n then
   begin
      i := m; j := n+1;
      k := Wk[m];
      repeat
         repeat
            i := i + 1;
         until (Wk[i] <= k) or (i = n);
         repeat
            j := j - 1;
         until Wk[j] >= k;
         if i < j then
            InterChange(Second+i,Second+j)
      until i >= j;
      InterChange(Second+m,Second+j);
      sort(m,j-1);
      sort(j+1,n);
   end;  (* end of if *)
end;


function Recover(i:integer;var Atoms,Leaves:integer):TwoPoints;
type
    StackPoint = ^StackNode;
    StackNode = record
        Tree : TwoPoints;
        Next : StackPoint;
    end;
var
    top : StackPoint;
    op  : byte;
     p  : TwoPoints;
    str : str4;
    ch  : char;

   procedure push(p:TwoPoints);
   var
       q : StackPoint;
   begin
      New(q);
      q^.Tree := p;
      q^.Next := Top;
      Top := q;
   end;

   function pop:TwoPoints;
   var
       q : StackPoint;
   begin
      if top <> nil then
      begin
         pop := top^.Tree;
         q := top;
         top := top^.Next;
         dispose(q);
      end
      else
         MessageDlg('Stack underfollow !! ',mtWarning,[],0);
   end;

begin
    top := nil;
    Atoms := 0;    Leaves := 0;
    FileName := 'T';
    if i >= 0 then
       OpenFile(0,i)
    else
    begin
       i := abs(i);
       OpenFile(-2,i);
    end;

    while not eof(Disk) do
    begin
       str := ReadSTR4;
       op := obtain(str);
       Atoms := Atoms + 1;
       if (op > piFunctionSet) then
          Leaves := Leaves + 1;
       New(p);
       p^.iData := op;
       if (op > piFunctionSet) then
       begin
          if P^.iData = piSetNo then
             read(Disk,ch,P^.R,ch,ch);
          p^.Left := nil;
          p^.Right := nil;
          push(p);
       end
       else
          if (op <= piBinarySet) then
          begin
             p^.right := pop;
             p^.left := pop;
             push(p);
          end
          else
          begin      (* op is unary *)
             p^.left := pop;
             P^.right := nil;
             push(p);
          end;
    end;   (* end of while not eof  *)

    recover := top^.Tree;
    closefile(Disk);
end;   (* end of recover *)


procedure WriteTree(i:integer);
begin
   FileName := 'T';
   OpenFile(1,i);
   This := Queue;
   while This <> nil do
   begin
      write(Disk,ElementsTable[This^.iData]);
      if This^.iData = piSetNo then
         write(Disk,'(',This^.R:4:2,' )');
      This := This^.Next;
   end;
   closefile(Disk);
end;


procedure ComputePstar;
var
    i : integer;
    Bi,Oi,Vt : extended;
    temp1,temp2 : extended;
    RanData : TextFile;
    DstarVar : integer;
begin
   Bi := 0.0;
   Oi := 0.0;
   BuyNo := 0;
   SellNo := 0;
   BeSNo := 0;

   if Gen >= 1 then
   begin
      for i := 1 to Individual do
      begin
         if (H[Second+i]-H[First+i]) > 0 then
         begin
            Bi := Bi + (H[Second+i]-H[First+i]);
            BuyNo := BuyNo + 1;
         end
         else
         if (H[Second+i]-H[First+i]) < 0 then
         begin
            Oi := Oi + Abs(H[Second+i]-H[First+i]);
            SellNo := SellNo + 1;
         end
         else
         begin
            BeSNo := BeSNo + 1;
         end;
      end;
      Tbid := Bi;
      TOffer := Oi;

      if Bi <= Oi then
         Vt := Bi
      else
         Vt := Oi;

      if (Bi <> 0.0) and (Oi <> 0.0) then
      begin
        for i := 1 to Individual do
        begin
           if (H[Second+i]-H[First+i]) > 0 then
              H[Second+i] := H[First+i] + (Vt/Bi)*(H[Second+i]-H[First+i])
           else
           begin
             if (H[Second+i]-H[First+i]) < 0 then
                H[Second+i] := H[First+i] + (Vt/Oi)*(H[Second+i]-H[First+i])
             else
                H[Second+i] := H[First+i];
           end;

           if Abs(H[Second+i]) <= 0.00001 then
              H[Second+i] := 0.0;

           M[Second+i] := W[First+i] - X[1]*H[Second+i];
           if M[Second+i] <= 0.00001 then
              M[Second+i] := 0.0
        end;
      end
      else
      begin
        for i := 1 to Individual do
        begin
          H[Second+i] := H[First+i];

          M[Second+i] := W[First+i] - X[1]*H[Second+i];
          if M[Second+i] <= 0.00001 then
             M[Second+i] := 0.0;
        end;
      end;

      temp1 := (Bi-Oi)/50000;
      temp2 := (Bi-Oi)/50000;
//    Form1.edtMsg.lines.add('HyperTan =  '+Floattostr(temp1));
      if Bi >= Oi then
        Pstar := X[1]*(1+((Exp(temp1)-Exp(-temp1))/(Exp(temp1)+Exp(-temp1))))
      else
        Pstar := X[1]*(1+((Exp(temp2)-Exp(-temp2))/(Exp(temp2)+Exp(-temp2))));

      if Pstar <= 0.0 then
         Pstar := 0.0;
      Randomize;
   end;

   assignfile(RanData,FilePath1+'\Dstar.txt');
   reset(RanData);
   for i := 1 to Gen do
     readln(RanData,Dstar);
   closefile(RanData);

// Randomize;
// DstarVar := Random(2);
// if DstarVar = 0 Then
//    Dstar := 10.0 + (Random(500)/100)
// else
//    Dstar := 10.0 - (Random(500)/100);
end;


(* Return the file index of tree *)
function ChoiceTree:integer;
var
    i, j : integer;
begin
// i := Random(Individual-33) + 33 + 1;
// j := Random(Individual-33) + 33 + 1;
   i := Random(Individual) + 1;
   j := Random(Individual) + 1;

   if Wk[i] >= Wk[j] then
       result := First+i
   else
       result := First+j;
end;


function ChoiceTreeFromLib:integer;
var
    i, j : integer;
begin
// i := Random(Individual-33) + 33 + 1;
// j := Random(Individual-33) + 33 + 1;
   i := Random(Individual) + 1;
   j := Random(Individual) + 1;

   if Mape[i] <= Mape[j] then
       Result := First+i
   else
       Result := First+j;
end;


function Reproduction(n:integer):integer;
var
    i,j : integer;
    Model : integer;
    Hi : real;
    My,Mx,Mxt,Mi : Matrix23;
    sItem,sAdd,sTree : string;
    t1, t2 : integer;
begin
   result := -1;
   if n = -1 then
   begin
      j := 1;
      for i := 12 to WinSize+11 do
      begin
         My[j,1] := DimA[i];
         Mx[1,j] := 1.0;
         j := j + 1;
      end;

      for Model := 1 to 11 do
      begin
        j := 1;
        for i := 12-Model to WinSize + 11 - Model do
        begin
           Mx[Model+1,j] :=DimA[i];
           j := j + 1;
        end;
         Mxt := MatrixTrans(Model+1,WinSize,Mx);
         Mi := MatrixMul(Model+1,Winsize,Mx,Winsize,Model+1,Mxt);
         Mi := MatrixInverse(Model+1,Model+1,Mi);
         Mi := MatrixMul(Model+1,Model+1,Mi,Model+1,Winsize,Mx);
         Mi := MatrixMul(Model+1,Winsize,Mi,Winsize,1,My);

//       for n := (Model-1)*3 + preserve +1 to Model*3+Preserve do
         for n := (Model-1)*3+1 to Model*3 do
         begin
            sAdd := '';
            sTree := '';
            for i := 0 to Model do
            begin
               sItem := 'R(' + floattostrf(Mi[i+1,1],ffGeneral,7,4) + ' )';
               if i <> 0 then
               begin
                  if i <= 10 then
                     sItem := sItem+'X0'+inttostr(i-1) + 'Y0'+Inttostr(i-1)+'+*'
                  else
                     sItem := sItem+'X'+inttostr(i-1) + 'Y'+Inttostr(i-1)+'+*';
                  sAdd := sAdd + '+';
               end;
               sTree := sTree + sItem;
            end;
            sTree := sTree + sAdd;

            FileName := 'T';
            OpenFile(1,Second+n);
            write(Disk,sTree );
            closefile(Disk);
            root := Recover(Second+n,t1,t2);
            DepthTable[n] := Depth2(root);
            NodeTable[n] := Count_Node(root);
            freetree(root);
            ReadTree(Second+n);
            ComputeHi(n,Hi);
            H[Second+n] := Hi;
            FreeQueue(Queue);
         end;
      end;
      exit;
   end
   else
   begin
      if n >= 0 then
      begin
         result := ChoiceTree;
         ReadTree(result);
      end
      else
      begin
         result := ChoiceTreeFromLib;
         ReadTree2(result);
         n := abs(n+1);
      end;
      WriteTree(Second+n);
      FreeQueue(Queue);
   end;
end;   (* end of Reproduction *)


procedure Mutate0(i:integer);
var
    n, Ran : integer;
begin
   if i >= 0 then
   begin
      n := ChoiceTree;
      ReadTree(n);
   end
   else
   begin
      n := ChoiceTreeFromLib;
      ReadTree2(n);
      i := abs(i);
   end;

   This := Queue;
   while This <> nil do
   begin
      Ran := Random(1000)+1;
      if Ran <= MutProb then
      begin
         if (This^.iData <= piBinarySet) then
         begin
            This^.iData := Random(piBinarySet) + 1;
         end
         else
            if (This^.iData > piBinarySet) and (This^.iData <=piFunctionSet) then
            begin
               This^.iData := Random(piUnarySet) + piBinarySet + 1;
            end
            else  (* In TerminalSet  *)
            begin
               This^.iData := Random(piTerminalSet) + piFunctionSet + 1;
               if This^.iData = piSetNo then
               begin
                  This^.R := Random(10)+Random(100)/100;
                  Ran := Random(2);
                  if Ran = 1 then
                     This^.R := (-1) * This^.R;
               end
               else
                  This^.R := 111.0;
            end;     (* In Terminal Set *)
      end;  (* if Ran <= MutProb *)
      This := This^.Next;
   end;   (* while This <> nil *)
   WriteTree(Second+i);
   FreeQueue(Queue);
end;    (*  end of Mutate *)


function Depth(T:TwoPoints;n:integer):integer;
var
    l, r : integer;
begin
   if (T <> nil) and (n < DepthLimit+2) then
   begin
      n := n + 1;
      l := Depth(T^.Left,n);
      r := Depth(T^.Right,n);
      if l >= r then
         Depth := l
      else
         Depth := r;
   end
   else
      Depth := n;
end;   (* end of Depth *)


procedure CutTree(var Count:integer;var T,Here:TwoPoints;var Turn:integer);
begin
   if (Count > 1) and (T <> nil) then
   begin
      Here := T;   Turn := 0;
      Count := Count - 1;
      CutTree(Count,T^.Left,Here,Turn);
      if (Count = 1) and (Turn = 0) then
         if Here^.Left <> nil then
            Count := 0;
      if (Count >= 1) and (T^.Right <> nil) then
      begin
         Turn := 1;
         if Count > 1 then
           CutTree(Count,T^.Right,Here,Turn)
         else
         begin
            Here := T;
            Count := 0;
         end;
      end;
   end;
end;   (* end of CutTree *)


function Cut(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
   Count := Random(Atoms)+1;
   if Count = 1 then
   begin
      Here := T;
      Turn := 2;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      CutTree(Count,T,Here,Turn);
   end;
   Cut := Here;
end;


procedure Mutate1(i:integer);
var
    n, Ran, d : integer;
    Atoms, Turn,Leaves : integer;
    t, s, Here, Tree   : TwoPoints;
begin
   if i >= 0 then
   begin
      n := ChoiceTree;
      Tree := Recover(n,Atoms,Leaves);
   end
   else
   begin
      n := ChoiceTreeFromLib;
      Tree := Recover(-n,Atoms,Leaves);
      i := abs(i);
   end;

   Ran := Random(1000)+1;

   if Ran <= MutProb then
   begin
      MakeGrowTree(t,1,6);
      Here := Cut(Tree,Atoms,Turn);
      case turn of
      0 : begin
             s := Here^.left;
             Here^.left := t;
             FreeTree(s);
          end;
      1 : begin
             s := Here^.Right;
             Here^.Right := t;
             FreeTree(s);
          end;
      2 : begin
             FreeTree(Tree);
             Tree := t;
          end;
      end;
      d := Depth(Tree,0);

      if d > DepthLimit then
      begin
         FreeTree(Tree);
         ReadTree(n);
         WriteTree(Second+i);
         FreeQueue(Queue);
      end
      else
      begin
         StoreTree(Tree,Second+i);
         FreeTree(Tree);
      end
   end
   else
   begin
      ReadTree(n);  (* Read file of tree i and build a queue list *)
      WriteTree(Second+i);
      FreeQueue(Queue);
   end;
end;    (*  end of Mutate0 *)


procedure Preorder(var Count:integer;T:TwoPoints;var Here:TwoPoints;var Turn:integer);
begin
   Here := T;
   if T <> nil then
      if (Count > 1) and (T^.iData <= piFunctionSet) then
      begin
         Here := T;   Turn := 0;
         Count := Count - 1;
         Preorder(Count,T^.Left,Here,Turn);
         if (Count = 1) and (Turn = 0) then
            if Here^.Left <> nil then
               if Here^.Left^.iData <= piFunctionSet then
                  Count := 0;
         if t^.Right <> nil then
            if (Count >= 1) and (T^.Right^.iData <= piFunctionSet) then
            begin
               Turn := 1;
               if Count > 1 then
                  Preorder(Count,T^.Right,Here,Turn)
               else
               begin
                  Here := T;
                  Count := 0;
               end;
            end;
      end;
end;   (* end of Preorder *)


procedure GetLeave(var Count:integer;T:TwoPoints;var Here:TwoPoints;var Turn:integer);
begin
   Here := nil;
   if T <> nil then
      if (T^.iData <= piFunctionSet) and (Count > 0) then
      begin
         if T^.Left <> nil then
            if T^.Left^.iData > piFunctionSet then
            begin
               Count := Count - 1;
               if Count = 0 then
               begin
                 Turn := 0;
                 Here := T;
               end;
            end
            else
               GetLeave(Count,T^.Left,Here,Turn);

         if Count > 0 then
            if T^.Right <> nil then
               if T^.Right^.iData > piFunctionSet then
               begin
                  Count := Count - 1;
                  if Count = 0 then
                  begin
                     Turn := 1;
                     Here := T;
                  end;
               end
               else
                  GetLeave(Count,T^.Right,Here,Turn);
      end;
end;


function CutLeave(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
 Count := Random(Atoms)+1;

   if T^.iData > piFunctionSet then
   begin
      Here := T;
      Turn := -1;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      GetLeave(Count,T,Here,Turn);
   end;
   CutLeave := Here;
end;


function CutPoint(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
    Count := Random(Atoms)+1;

   if Count = 1 then
   begin
      Here := T;
      Turn := -1;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      Preorder(Count,T,Here,Turn);
   end;
   CutPoint := Here;
end;


procedure Crossover(i:integer);
var
    Tree1, Tree2, q   : TwoPoints;
    Here1, Here2      : TwoPoints;
    k, Atoms1, Atoms2 : integer;
    Leaves1, Leaves2  : integer;
    Turn1,Turn2,Mode  : integer;
    n1, n2, d1, d2    : integer;
begin
   if i >= 0 then
   begin
      n1 := ChoiceTree;
      Tree1 := Recover(n1,Atoms1,Leaves1);
      n2 := ChoiceTree;
      Tree2 := Recover(n2,Atoms2,Leaves2);
   end
   else
   begin
      n1 := ChoiceTreeFromLib;
      Tree1 := Recover(-n1,Atoms1,Leaves1);
      n2 := ChoiceTreeFromLib;
      Tree2 := Recover(-n2,Atoms2,Leaves2);
      i := abs(i);
   end;

   Mode := Random(10);
   if Mode < LeaveProb then
      Here1 := CutLeave(Tree1,Leaves1,Turn1)   (* Choice Leaves *)
   else
      Here1 := CutPoint(Tree1,Atoms1-Leaves1,Turn1);
   Mode := Random(10);
   if Mode < LeaveProb then
      Here2 := CutLeave(Tree2,Leaves2,Turn2)
   else
      Here2 := CutPoint(Tree2,Atoms2-Leaves2,Turn2);
                            (* Here1 or Here2  points to root of tree. *)
   if Turn1 = -1 then       (* So, We make a dummy head to  point it.  *)
   begin                    (* This dummy head's Data is "Plus", and   *)
      New(Tree1);           (* Right is nil, but it's Left points to   *)
      Tree1^.iData := 1;    (* Here1 or Here2.                         *)
      Tree1^.Right := nil;
      Tree1^.Left := Here1;
      Here1 := Tree1;
      Turn1 := 0;
   end;
   if Turn2 = -1 then
   begin
      New(Tree2);
      Tree2^.iData := 1;
      Tree2^.Right := nil;
      Tree2^.Left := Here2;
      Here2 := Tree2;
      Turn2 := 0;
   end;
   if Turn1 = 0 then
   begin
      q := Here1^.Left;
      if Turn2 = 0 then                (* Turn1=0 and Turn2=0 *)
      begin
         Here1^.Left := Here2^.Left;
         Here2^.Left := q;
      end
      else                              (* Turn1=0 and Turn2=1 *)
      begin
         Here1^.Left  := Here2^.Right;
         Here2^.Right := q;
      end
   end
   else    (* Turn1 = 1  *)
   begin
      q := Here1^.Right;
      if Turn2 = 0 then                   (* Turn1=1 and Turn2=0 *)
      begin
         Here1^.Right := Here2^.Left;
         Here2^.Left  := q;
      end
      else                                 (* Turn1=1 and Turn2=1 *)
      begin
         Here1^.Right := Here2^.Right;
         Here1^.Right := q;
      end;
   end;
   if (Tree1^.iData = 1) and (Tree1^.Right = nil) then
      Tree1 := Tree1^.Left;
   if (Tree2^.iData = 1) and (Tree2^.Right = nil) then
      Tree2 := Tree2^.Left;

   d1 := Depth(Tree1,0);   d2 := Depth(Tree2,0);
   k := 0;
   if d1 <= DepthLimit then
   begin
      StoreTree(Tree1,Second+i);
      FreeTree(Tree1);
   end
   else
      if (d2 <= DepthLimit) then
      begin
         StoreTree(Tree2,Second+i);
         FreeTree(Tree2);
      end
      else
      begin
         ReadTree(n1);
         WriteTree(Second+i);
         FreeQueue(Queue);
      end;
end;


procedure StoreData;
var
    i : integer;
    sLine,FileName : string;
    F : TextFile;
begin
   VarNode := 0.0; VarDepth := 0.0; VarHi := 0.0; VarVijs := 0.0;
   TWealth := 0.0;  TWealth1 := 0.0;  TWealth2 := 0.0;
   TWealth3 := 0.0; TWealth4 := 0.0;  TWealth5 := 0.0;
   AveDepth := 0.0; AveNode := 0.0; AveVijs := 0.0; AveHi := 0.0;
   AveSigma2 := 0.0; VarSigma2 := 0.0;

   for i := 1 to Individual do
   begin
      AveDepth := AveDepth + DepthTable[i];
      AveNode := AveNode + NodeTable[i];
      TWealth := TWealth + W[FileIndex[Second+i]];
      AveHi := AveHi + H[Second+i];
      AveVijs := AveVijs + Vijs[Second+i];
      AveSigma2 := AveSigma2 + Sigma[Second+i];
   end;
   AveDepth := AveDepth / Individual;
   AveNode := AveNode / Individual;
   AveHi := AveHi / Individual;
   AveVijs := AveVijs / Individual;
   AveSigma2 := AveSigma2 / Individual;

   for i := 1 to Individual do
   begin
     VarNode := VarNode  + Sqr(NodeTable[i]-AveNode);
    VarDepth := VarDepth + Sqr(DepthTable[i]-AveDepth);
       VarHi := VarHi    + Sqr(H[Second+i]-AveHi);
     VarVijs := VarVijs  + Sqr(Vijs[Second+i]-AveVijs);
   VarSigma2 := VarSigma2 + Sqr(Sigma[Second+i]-AveSigma2);
   end;
     VarNode :=   VarNode / Individual;
    VarDepth :=  VarDepth / Individual;
       VarHi :=     VarHi / Individual;
     VarVijs :=   VarVijs / Individual;
   VarSigma2 := VarSigma2 / Individual;

   for i := 1 to 100 do   // Based on Income index Definition
   begin
      TWealth1 := TWealth1 + W[FileIndex[Second+i]];
      TWealth2 := TWealth2 + W[FileIndex[Second+i+100]];
      TWealth3 := TWealth3 + W[FileIndex[Second+i+200]];
      TWealth4 := TWealth4 + W[FileIndex[Second+i+300]];
      TWealth5 := TWealth5 + W[FileIndex[Second+i+400]];
   end;
   TWealth1 := TWealth1 / TWealth;
   TWealth2 := TWealth2 / TWealth;
   TWealth3 := TWealth3 / TWealth;
   TWealth4 := TWealth4 / TWealth;
   TWealth5 := TWealth5 / TWealth;

   write(out1,Gen:5,'  ',Pstar:14:5,'  ',Dstar:5:2,'  ',BuyNo:3);
   write(out1,'    ',SellNo:3,'  ',BeSNo:5,'  ',TBid:12:5,'  ',TOffer:9:5);
   write(out1,'  ',Twealth1:7:5,'   ',Twealth2:7:5,'   ',Twealth3:7:5);
   write(out1,'   ',Twealth4:7:5,'   ',Twealth5:7:5,'  ',AveDepth:8:5);
   write(out1,'   ',AveNode:9:5,'  ',AveHi:8:5,'  ',AveVijs:10:5,'  ',AveSigma2:12:5);
   write(out1,'  ',VarDepth:9:5,'   ',VarNode:10:5,'  ',VarHi:9:5);
   write(out1,'  ',VarVijs:12:5,'   ',VarSigma2:12:5,'    ',SucceedNo:3,'           ',MartingaleNo:3);
   writeln(out1,'           ',VisitGenNo:3,'          ',VisitLibNo:3);

if (((Gen = 1) or ((Gen mod 500) = 0))) then
begin
  writeln(out2,'Generation:',GEN);
  for i := 1 to Individual do
  begin
     write(Out2,Y[1]-Vijs[First+i]:15:5,'  ',H[Second+i]:10:5,'  ',Sigma[Second+i]:10:5,'  ',Vijs[Second+i]:15:5,'  ',Vij1[i]:15:5,'   ');
     FileName := 'T'+Copy(trim(inttostr(10000 + Second+i)),2,4)+'.dat';
     case (Second+i) div 100 of
       0: FileName := FilePath1 +'0\'+ FileName;
       1: FileName := FilePath1 +'1\'+ FileName;
       2: FileName := FilePath1 +'2\'+ FileName;
       3: FileName := FilePath1 +'3\'+ FileName;
       4: FileName := FilePath1 +'4\'+ FileName;
       5: FileName := FilePath1 +'5\'+ FileName;
       6: FileName := FilePath1 +'6\'+ FileName;
       7: FileName := FilePath1 +'7\'+ FileName;
       8: FileName := FilePath1 +'8\'+ FileName;
       9,10: FileName := FilePath1 +'9\'+ FileName;
     end;
     assignfile(F,FileName);
     reset(F);
     while not eof(f) do
     begin
        readln(F,sLine);
        write(out2,sline);
     end;
     writeln(out2);
     closefile(F);
  end;
end;
end;


procedure InorderOut(Tree:TwoPoints);
var
    Jump : boolean;
begin
   Jump := false;
   if Tree <> nil then
   begin
      if Tree^.Left <> nil then
      begin
         if Tree^.Right <> nil then
            write(BestFile,'(')
         else
         begin
            if Tree^.iData = piSetNo then
               write(BestFile,Tree^.R:4:2)
            else
               write(BestFile,ElementsTable[Tree^.iData]);
            Jump := True;
         end;
         InorderOut(Tree^.Left)
      end;
      if Not Jump then
         if Tree^.iData = piSetNo then
            write(BestFile,Tree^.R:4:2)
         else
            write(BestFile,ElementsTable[Tree^.iData]);
      if Tree^.Right <> nil then
      begin
         InorderOut(Tree^.Right);
         write(BestFile,')');
      end;
   end;
end;


procedure StoreBestTree;
var
    Tree : TwoPoints;
    This : OnePoint;
    i,Best : integer;
    BestProfit : extended;
begin
   Best := 1;
   BestProfit := Profits[1];
   for i := 2 to Individual do
      if Profits[i] > BestProfit then
      begin
         Best := i;
         BestProfit := Profits[i];
      end;
    Best := Second + Best;
    FileName := 'Best';
    OpenFile(2,Gen+1);
    writeln(BestFile,'The Wealth = ',W[Best+Second]:15:5);
    writeln(BestFile,'The Profit = ',Profits[Best]);
    writeln(BestFile,'The Amount of Money = ',M[Best+Second]:15:5);
    writeln(BestFile,'The Share of Stock = ',H[Best+Second]:15:5);
   write(BestFile,'The Inorder form of GEN',Gen+1:3,'  ==> ');
   Tree := Recover(Best,i,i);   (* i give up *)
   InorderOut(Tree);
   writeln(BestFile);
   write(BestFile,'The PostOrder form of GEN',Gen+1:3,'  ==> ');
   FreeTree(Tree);

   ReadTree(Best);
   This:= Queue;
   while This <> nil do
   begin
      write(BestFile,ElementsTable[This^.iData]);
      if This^.iData = piSetNo Then
         write(BestFile,'(',This^.R:4:2,' )');
      This := This^.Next;
   end;
   FreeQueue(Queue);
   writeln(BestFile);
   closefile(BestFile);
end;


procedure Init;
var
   i : integer;
begin
   for i := 1 to Individual do
   begin
      Sigma[First+i] := 4.0;
      Vijs[First+i] := Y[1];
      H[First+i] := H0;
      M[First+i] := M0;
      W[First+i] := M0 + H0 * X[1];
      DepthTable[i] := 0;
      NodeTable[i] := 0;
      Wk[i] := 0.0;
      Wk2[i] := (Irate * M0 + H0 / Irate) * G_Gap;
   end;
end;


procedure TForm1.OutPut_Msg(sMsg:string);
begin
   if Out_Count < 200 then
      edtMsg.lines.add(inttostr(Out_Count) + ': '+sMsg)
   else
      edtMsg.Lines[Out_Count Mod 200] := inttostr(Out_Count) + ': '+sMsg;
   Out_Count := Out_Count + 1;
   application.ProcessMessages;
end;


procedure DumpData;
var
    i : integer;
    Myini : TIniFile;
    sGen,sLine : string;
begin
   Myini:=TIniFile.Create(sInifile);
   Myini.WriteString('Common','Stop_No',inttostr(Gen));
   Myini.Free;
   if fileExists(FilePath1+'\Backup1.txt') then
   begin
      sGen := 'GEN='+inttostr(Gen);
      assignfile(Backup,FilePath1+'\Backup1.txt');
      reset(Backup);
      readln(Backup,sLine);
      closefile(Backup);
      if Pos(sGen,sLine) > 0 then
         Exit;

      deletefile(FilePath1+'\Backup11.txt');
      application.ProcessMessages;
      if fileExists(FilePath1+'\Backup1.txt') then
         renameFile(FilePath1+'\Backup1.txt',FilePath1+'\Backup11.txt');
      application.ProcessMessages;
   end;

   assignfile(Backup,FilePath1+'\Backup1.txt');
   rewrite(Backup);
   writeln(Backup,sGen,'  First='+inttostr(First),'  Second='+inttostr(Second));

   for i := 1 to piX_No do
      writeln(Backup,X[i]:12:5);

   for i := 1 to piY_No do
      writeln(Backup,Y[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,Sigma[i]:12:5);

   for i := 1 to Individual do
       writeln(Backup,Profits[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,H[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,M[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,W[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,Vijs[i]:12:5);

   for i := 1 to FileAmount do
       writeln(Backup,FileIndex[i]);

   for i := 1 to Individual do
       writeln(Backup,DepthTable[i]);

   for i := 1 to Individual do
       writeln(Backup,NodeTable[i]);

   writeln(Backup,Pstar:12:5, Dstar:12:5);
   writeln(Backup,First,' ',Second);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimA[i]);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimB[i]);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimC[i]);

   for i := 1 to Individual do
       writeln(Backup,Wk[i]);

   for i := 1 to Individual do
       writeln(Backup,Wk2[i]);

   writeln(Backup,BuyNo,' ',SellNo,' ',BeSNo,' ',Out_Count,' ',TBid,' ',TOffer);
   writeln(Backup,TWealth,' ',TWealth1,' ',TWealth2,' ',TWealth3,' ',TWealth4,' ',TWealth5);
   writeln(Backup,AveDepth,' ',AveNode,' ',AveHi,' ',AveVijs,' ',AveSigma2,' ',VarDepth,' ',VarNode);
   writeln(Backup,VarHi,' ',VarVijs,' ',VarSigma2,' ',SucceedNo,' ',MartingaleNo,' ',VisitGenNo:3,' ',VisitLibNo);
   closefile(Backup);
end;


procedure ReadOldData;
var i : integer;
    sItem : string;
begin
   if piExVar_No > 0 then
   begin
      assignFile(ExData,ExDataFile);
      reset(ExData);
      for i := 1 to Gen - 1 do
      begin
         readln(ExData);
         if eof(ExData) then
         begin
            closefile(ExData);
            reset(ExData);
         end;
      end;

      for i := 1 to piExVar_No do
          read(ExData,ExVar[i]);
      readln(ExData);
      if eof(ExData) then
      begin
         closefile(ExData);
         Reset(ExData);
      end;
   end;

   assignfile(Backup,FilePath1+'\Backup1.txt');
   reset(Backup);
   readln(Backup,sItem);

   for i := 1 to piX_No do
       readln(Backup,X[i]);

   for i := 1 to piY_No do
       readln(Backup,Y[i]);

   for i := 1 to FileAmount do
       readln(Backup,Sigma[i]);

   for i := 1 to Individual do
       readln(Backup,Profits[i]);

   for i := 1 to FileAmount do
       readln(Backup,H[i]);

   for i := 1 to FileAmount do
       readln(Backup,M[i]);

   for i := 1 to FileAmount do
       readln(Backup,W[i]);

   for i := 1 to FileAmount do
       readln(Backup,Vijs[i]);

   for i := 1 to FileAmount do
       readln(Backup,FileIndex[i]);

   for i := 1 to Individual do
       readln(Backup,DepthTable[i]);

   for i := 1 to Individual do
       readln(Backup,NodeTable[i]);

   readln(Backup,Pstar,Dstar);
   readln(Backup,First,Second);

   for i := 1 to WinSize+11 do
       readln(Backup,DimA[i]);

   for i := 1 to WinSize+11 do
       readln(Backup,DimB[i]);

   for i := 1 to WinSize+11 do
       readln(Backup,DimC[i]);

   for i := 1 to Individual do
       readln(Backup,Wk[i]);

   for i := 1 to Individual do
       readln(Backup,Wk2[i]);

   readln(Backup,BuyNo,SellNo,BeSNo,Out_Count,TBid,TOffer);
   readln(Backup,TWealth,TWealth1,TWealth2,TWealth3,TWealth4,TWealth5);
   readln(Backup,AveDepth,AveNode,AveHi,AveVijs,AveSigma2,VarDepth,VarNode);
   readln(Backup,VarHi,VarVijs,VarSigma2,SucceedNo,MartingaleNo,VisitGenNo,VisitLibNo);
   closefile(Backup);
end;


procedure Wait;
var
   n : longint;
begin
   try
     while Fileexists(psWaitFile) do
     begin
        if ReadINI('Common','Exit',sInifile) = '1' then
        begin
           pbExit := true;
           exit;
        end;
        for n := 1 to 10000 do
          application.ProcessMessages;
     end;
   except

      for n := 1 to 100000 do
        application.ProcessMessages;
      if ReadINI('Common','Exit',sInifile) = '1' then
      begin
         pbExit := true;
         exit;
      end;
   end;
end;


procedure Compare(i:integer;var Wk_New:extended);
var
    ch : string[1];
    Value : extended;
    fMean,AveSigma,Hi : real;
    j,u,iFirst,iSecond,FileIndex_old : integer;
    H_old,M_old,W_old,Sigma_Old,Vijs_old : extended;
begin
   if G_Gap = 1 then
      ch := 'k'
   else
      ch := '1';

   H_old := H[First+i];
   M_old := M[First+i];
   W_old := W[First+i];
   Sigma_old := Sigma[First+i];
   Vijs_old := Vijs[First+i];
   FileIndex_old := FileIndex[First+i];

   assignfile(History,FilePath1 + '\History'+ch+'.h');
   reset(History);
   for j := 1 to i do
   readln(History,H[First+i]);
   closefile(History);

   assignfile(History,FilePath1 + '\History'+ch+'.m');
   reset(History);
   for j := 1 to i do
   readln(History,M[First+i]);
   closefile(History);

   assignfile(History,FilePath1 + '\History'+ch+'.w');
   reset(History);
   for j := 1 to i do
   readln(History,W[First+i]);
   closefile(History);

   assignfile(History,FilePath1 + '\History'+ch+'.s');
   reset(History);
   for j := 1 to i do
   readln(History,Sigma[First+i]);
   closefile(History);

   assignfile(History,FilePath1 + '\History'+ch+'.v');
   reset(History);
   for j := 1 to i do
   readln(History,Vijs[First+i]);
   closefile(History);

   iFirst := First;
   iSecond := Second;

   Wk_New := 0.0;
   for u := (111 - G_Gap) to 110 do
   begin
      ComputeVij(i,u,Vijs[Second+i]);

      fMean:=0.0; AveSigma := 0.0;
      for j := (u-9) to u do
         fMean := fMean + (DimA[j]);
      fMean := fMean/10;

      for j := (u-9) to u do
         AveSigma := AveSigma + Sqr(DimA[j] - fMean);
      AveSigma := AveSigma/9;

      Sigma[First+i] := AveSigma;
      Sigma[Second+i] := (1-Theta3)*Sigma[First+i]+Theta3*Sqr(DimC[u]-Vijs[First+i]);

      Hi := (Vijs[Second+i]-DimB[u]*(1+Irate))/(Lambda*Sigma[Second+i]);
      if Hi <= 0.0 then
         Hi := 0.0
      else
      begin
        if Abs(Hi) <= 0.00001 then
           Hi := 0.0
        else
        begin
          if (W[First+i] - DimB[u]*Hi) <= 0.0 then
             Hi := W[First+i]/DimB[u];
        end;
      end;
      M[Second+i] := W[First+i] - DimB[u]*Hi;
      if Abs(M[Second+i]) < 0.00001 then
         M[Second+i] := 0.0;

      Value := Irate*M[Second+i] + Hi*(DimC[u+1]-DimB[u]);
      Wk_New := Wk_New + Value;
      W[Second+i] := W[First+i] + Value;

      First := Abs(Individual-First);
      Second := Abs(Individual-Second);
   end;
   First := iFirst;
   Second := iSecond;

   H[First+i] := H_old;
   M[First+i] := M_old;
   W[First+i] := W_old;
   Sigma[First+i] := Sigma_old;
   Vijs[First+i] := Vijs_old;
   FileIndex[First+i] := FileIndex_old;
end;


procedure Selection(i:integer);
var
   bExit : boolean;
   iTry,k,u,t1,t2,FileIndex_old : integer;
   Vij2,ESum1,ESum2 : extended;
   M_old,W_Old,Vijs_old : extended;
   Value,H_old,Sigma_Old,Hi : real;
   Wk_New : extended;
begin
   iTry := 0;
   repeat
      Value := Random(100);
      if (Value >= LibraryProb) or (Gen <= 2) then
      begin
         VisitGen := 1;
         Value := Random(Individual)+1;
         if Value <= Reproduct then
         begin
            k := Reproduction(i);
            if Wk[i] <= Wk[k] then
                bExit := true;
         end
         else
         begin
            if Value <= Reproduct+Mutation then
            begin
               if Mutmode = 0 Then
                  Mutate0(i)
               else
                  Mutate1(i);
            end
            else
               Crossover(i);
            ReadTree(Second+i);
            Compare(i,Wk_New);
            if Wk_New > Wk[i] then
               bExit := true;
         end;
      end
      else
      begin
         VisitLib := 1;

         Value := Random(Reproduct);  // Trees from Library are by reproduction
         if Value <= Reproduct then
         begin
            k := Reproduction(-(i+1));
            if Mape[i] >= Mape[k] then
                bExit := true;
         end
         else
         begin
            if Value <= Reproduct+Mutation then
            begin
               if Mutmode = 0 Then
                  Mutate0(-i)
               else
                  Mutate1(-i);
            end
            else
               Crossover(-i);
         end;

         ReadTree(Second+i);
         H_old := H[First+i];
         M_old := M[First+i];
         W_old := W[First+i];
         Sigma_old := Sigma[First+i];
         Vijs_old := Vijs[First+i];
         FileIndex_old := FileIndex[First+i];
         ESum1 := 0.0;
         ESum2 := 0.0;
         bExit := false;
         for u := 111 downto 102 do   // Compute tree selected from library
         begin
            ComputeVij(i,u,Vij2);
            ESum1 := ESum1 + Abs((Vij2-DimA[u])/DimA[u]);
         end;
         H[First+i] := H_old;
         M[First+i] := M_old;
         W[First+i] := W_old;
         Sigma[First+i] := Sigma_old;
         Vijs[First+i] := Vijs_old;
         FileIndex[First+i] := FileIndex_old;
         FreeQueue(Queue);

         H_old := H[First+i];
         M_old := M[First+i];
         W_old := W[First+i];
         Sigma_old := Sigma[First+i];
         Vijs_old := Vijs[First+i];
         FileIndex_old := FileIndex[First+i];

         ReadTree(First+i);
         ESum2 := 0.0;
         for u := 111 downto 102 do
         begin
            ComputeVij(i,u,Vij2);
            ESum2 := ESum2 + Abs((Vij2-DimA[u])/DimA[u]);
         end;
         H[First+i] := H_old;
         M[First+i] := M_old;
         W[First+i] := W_old;
         Sigma[First+i] := Sigma_old;
         Vijs[First+i] := Vijs_old;
         FileIndex[First+i] := FileIndex_old;

         if ESum1 < ESum2 then  // The tree selected from library is better
            bExit := true;
      end;
      FreeQueue(Queue);
      iTry := iTry + 1;
   until bExit or (iTry >= TryNo);

   if bExit then
   begin
      Succeed := 1;
      ReadTree(Second+i);
      ComputeHi(i,Hi);
      H[Second+i] := Hi;
      FreeQueue(Queue);
      root := Recover(Second+i,t1,t2);
      DepthTable[i] := Depth2(root);
      NodeTable[i] := Count_Node(root);
      freetree(root);
   end
   else
      begin
         Succeed := 0;
         ReadTree(First+i);
         ComputeHi(i,Hi);
         H[Second+i] := Hi;
         WriteTree(Second+i);
         FreeQueue(Queue);
         root := Recover(Second+i,t1,t2);
         DepthTable[i] := Depth2(root);
         NodeTable[i] := Count_Node(root);
         freetree(root);
      end;
end;


procedure TForm1.ShowGraph(x:integer;y:extended);
var X_Value : real;
begin
   X_Value := x * 1.0;
   With Chart1 do
   begin
      With Series[0] do
      begin
         if x <= piBeginGen+500 then   //* The window size of the dynamic graph
            AddXY(X_Value,y,'',clRed)
         else
         begin
            try
               Delete(0)
            except
            end;
            AddXY(X_Value,Y,'',clRed);
         end;
      end;
   end;
   application.ProcessMessages;
end;


procedure TForm1.GetEnv(sIniFile:string);
var
   i,iPos : integer;
   sCode,sSet : string;
begin
   H0 := strtofloat(ReadINI('Stock_Market','H0',sIniFile));
   M0 := strtofloat(ReadINI('Stock_Market','M0',sIniFile));
   Irate := strtofloat(ReadINI('Stock_Market','Irate',sIniFile));
   Beta := strtofloat(ReadINI('Stock_Market','Beta',sIniFile));
   Full := strtoint(ReadINI('Traders','Full',sIniFile));
   Grow := strtoint(ReadINI('Traders','Grow',sIniFile));
   Reproduct := strtoint(ReadINI('Traders','Reproduct',sIniFile));
   Mutation := strtoint(ReadINI('Traders','Mutation',sIniFile));
   MutProb := strtofloat(ReadINI('Traders','MutProb',sIniFile));
   DepthLimit := strtoint(ReadINI('Traders','DepthLimit',sIniFile));
   LeaveProb := strtoint(ReadINI('Traders','LeaveProb',sIniFile));
   MaxExp := strtofloat(ReadINI('Traders','MaxExp',sIniFile));
   MutMode := strtoint(ReadINI('Traders','MutMode',sIniFile));
   G_Gap := strtoInt(ReadINI('Traders','G_Gap',sIniFile));
   LibraryProb := strtoInt(ReadINI('Traders','LibraryProb',sIniFile));
   Lambda := strtofloat(ReadINI('Traders','Lambda',sIniFile));
   Theta1 := strtofloat(ReadINI('Traders','Theta1',sIniFile));
   Theta2 := strtofloat(ReadINI('Traders','Theta2',sIniFile));
   Theta3 := strtofloat(ReadINI('Traders','Theta3',sIniFile));
   TryNo := strtoint(ReadINI('Traders','TryNo',sIniFile));

   sSet := uppercase(trim(ReadINI('Common','BinarySet',sIniFile)));
   i := 1;
   piBinarySet := 0;
   piSetNo := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piBinarySet := piBinarySet + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','UnarySet',sIniFile)));
   piUnarySet := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piUnarySet := piUnarySet + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;
   piFunctionSet := piUnarySet + piBinarySet;


   sSet := uppercase(trim(ReadINI('Common','X_Var',sIniFile)));
   piX_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piX_No := piX_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','Y_Var',sIniFile)));
   piY_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piY_No := piY_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','Extra_Var',sIniFile)));
   piExVar_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piExVar_No := piExVar_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   piTerminalSet := piX_No + piY_No + piExVar_No;
   psConst := uppercase(trim(ReadINI('Common','Const_Item',sIniFile)));
   if psConst <> '' then
   begin
      ElementsTable[i] := psConst;
      piSetNo := piSetNo + 1;
      piTerminalSet := piTerminalSet + 1;
   end;
end;


procedure TForm1.FormActivate(Sender:TObject);
label Exit_Point;
var
   ch : string[1];
   i,G,BestNo,t1,t2,n,iTry,u : integer;
   Vij,Vij2 : extended;
   Value,p,q,Hi : real;
   Myini : TIniFile;
   DimAFile,DimBFile,DimCFile : TextFile;
   bReplay,bOpenMapeOk : boolean;
begin
   sIniFile := ApPath + 'Ver2.ini';
   Myini := TIniFile.Create(sInifile);
   Myini.WriteString('Common','Running','1');
   Myini.WriteString('Common','Exit','0');
   Myini.Free;
   pbExit := false;
   pbCancel := false;
   Randomize;
   if not FileExists(sIniFile) then
   begin
      MessageDlg(sIniFile + ' Not Found!',mtError,[mbOK],0);
      application.Terminate;
      exit;
   end;
   GetEnv(sIniFile);
   DataFile := ReadINI('Common','Data_File',sIniFile);
   ExDataFile := ReadINI('Common','ExData_File',sIniFile);
   FilePath1 := ReadINI('Common','Tree_Path1',sIniFile);
   FilePath2 := ReadINI('Common','Tree_Path2',sIniFile);
   Out1Name := FilePath1 + '\Data1.txt';
   Out2Name := FilePath1 + '\Data2.txt';
   for i := 0 to 9 do
   begin
      if not DirectoryExists(FilePath1+trim(inttostr(i))) then
         mkdir(FilePath1 + trim(inttostr(i)));
      if not DirectoryExists(FilePath2+trim(inttostr(i))) then
         mkdir(FilePath2 + trim(inttostr(i)));
      application.ProcessMessages;
   end;
   GNumber := strtoint(ReadINI('Traders','Gen_No',sIniFile));
   GEN := strtoint(ReadINI('Common','Stop_No',sIniFile));
   piBeginGen := Gen;
   piDoDim := strtoint(ReadINI('Common','Do_Dim',sIniFile));

   if GEN = 0 then
   begin
      psWaitFile := FilePath1 + '\Gen0';
      bRePlay := false;
      assignfile(Data,DataFile);
      if piExVar_No > 0 then
      begin
         assignFile(ExData,ExDataFile);
         reset(ExData);
         for i := 1 to piExVar_No do
            read(ExData,ExVar[i]);
         readln(ExData);    
      end;
      assignfile(out1,Out1Name);
      assignfile(out2,Out2Name);
      rewrite(out1);
      rewrite(out2);
      reset(Data);

      for i := 1 to piX_No do
         read(Data,X[i]);
      readln(Data);
      for i := 1 to piY_No do
         read(Data,Y[i]);
      closefile(Data);

      if piDoDim = 0 then
      begin
         assignfile(DimAFile,ReadINI('Common','DimA_File',sIniFile));
         assignfile(DimBFile,ReadINI('Common','DimB_File',sIniFile));
         assignfile(DimCFile,ReadINI('Common','DimC_File',sIniFile));
         reset(DimAFile);
         reset(DimBFile);
         reset(DimCFile);
         for i := 1 to WinSize + 11 do
         begin
            readln(DimAFile,DimA[i]);
            readln(DimBFile,DimB[i]);
            readln(DimCFile,DimC[i]);
         end;
         closefile(DimAFile);
         closefile(DimBFile);
         closefile(DimCFile);
      end;
      write(out1,'GEN          P(t+1)    D(t+1) ');
      write(out1,'BuyNo SellNo BeSNo     TotalBid  TotalOffer ');
      write(out1,'Twealth1  Twealth2  Twealth3  Twealth4  Twealth5');
      write(out1,'  AveDepth    AveNode   AveHi      AveVijs       AveSigma');
      write(out1,'    VarDepth     VarNode    VarHi');
      write(out1,'        VarVijs        VarSigma    SucceedNo     MartingaleNo');
      writeln(out1,'  VisitGenNo   VisitLibNo');

      if (LibraryProb <> 0) then
         WinExec(pchar(ApPath+'Ver2Ap.exe'),0);
      Gen := 1;
      First := 500;
      Second := 0;
      edtMsg.Lines[0] := 'Processing Generation        1 ';
      Out_Count := 1;
      application.ProcessMessages;
      Init;
      ComputeProfit;
      Generate(0);
      ComputePstar;
      OutPut_msg('P(t+1) =  '+Floattostr(Pstar));
      ShowGraph(Gen,Pstar);
      application.ProcessMessages;
      for i := 1 to Individual do
         FileIndex[i] := i;
      sort(1,Individual);
      StoreBestTree;
      StoreData;
   end
   else
   begin
      psWaitFile := FilePath1 + '\Gen'+inttostr(GEN-1);
      ReadOldData;
      assignfile(Data,DataFile);
      assignfile(out1,Out1Name);
      assignfile(out2,Out2Name);
      Append(out1);
      Append(out2);
      reset(Data);
      if piDoDim = 0 then
      begin
         assignfile(DimAFile,ReadINI('Common','DimA_File',sIniFile));
         assignfile(DimBFile,ReadINI('Common','DimB_File',sIniFile));
         assignfile(DimCFile,ReadINI('Common','DimC_File',sIniFile));
      end;
      Out_Count := 1;
      if (LibraryProb <> 0) then
         WinExec(pchar(ApPath+'Ver2Ap.exe'),0);
      bRePlay := true;
      application.ProcessMessages;
   end;

   while (GEN < GNumber) and (not pbExit) do
   begin
      i := Random(10000);
      u := Random(2);
      if u = 1 then
         RandSeed := strtoint(FormatDateTime('hhnnss',now))-i
      else
         RandSeed := strtoint(FormatDateTime('hhnnss',now))+i;

      if not bReplay then
      begin
         for i := piX_No downto 2 do
            X[i] := X[i-1];

         for i := piY_No downto 2 do
            Y[i] := Y[i-1];

         X[1] := Pstar;
         Y[1] := Pstar + Dstar;

         if piExVar_No > 0 then
         begin
            for i := 1 to piExVar_No do
               read(ExData,ExVar[i]);
            readln(ExData);
            if eof(ExData) then
            begin
               closefile(ExData);
               Reset(ExData);
            end;
         end;
      end
      else
         bReplay := false;
      DumpData;
  try

     if (Gen <= WinSize + 11) and (piDoDim <> 0) then
     begin
        DimA[Gen] := Pstar + Dstar;
        DimB[Gen] := X[1];
        DimC[Gen] := Y[1];
     end
     else
     begin
       for i := 1 to WinSize+11 - 1 do
       begin
          DimA[i] := DimA[i+1];
          DimB[i] := DimB[i+1];
          DimC[i] := DimC[i+1];
       end;
       DimA[WinSize+11] := Pstar + Dstar;
       DimB[WinSize+11] := X[1];
       DimC[WinSize+11] := Y[1];
     end;

   if (LibraryProb <> 0) then
   begin
       Wait;
       if pbExit then
          Goto Exit_Point;
      if Gen > 1 then
      begin
         assignfile(MapeFile,FilePath1 +'\Mape.txt');
         bOpenMapeOk := false;
         repeat
            try
               reset(MapeFile);
               bOpenMapeOk := true;
            except
            end;
            application.ProcessMessages;
         until bOpenMapeOk;
         for i := 1 to Individual do
            readln(MapeFile,Mape[i]);
         closefile(MapeFile);
      end;
   end;

   rewrite(Data);
   for i := 1 to piX_No do
      write(Data,X[i]:7:3,'  ');
   writeln(Data);
   for i := 1 to piY_No do
      write(Data,Y[i]:7:3,'  ');
   closefile(Data);

   rewrite(DimAFile);
   rewrite(DimBFile);
   rewrite(DimCFile);
   for i := 1 to WinSize + 11 do
   begin
      writeln(DimAFile,DimA[i]);
      writeln(DimBFile,DimB[i]);
      writeln(DimCFile,DimC[i]);
   end;
   closefile(DimAFile);
   closefile(DimBFile);
   closefile(DimCFile);

   if (LibraryProb <> 0) then
   begin
      psWaitFile :=  FilePath1 + '\Gen' + inttostr(Gen);
      assignfile(fWait,psWaitFile);
      rewrite(fWait);
      closefile(fWait);
   end;
//------------------------------------------------------------------------
     Gen := Gen + 1;
     OutPut_msg('Processing Generation     '+ inttostr(Gen));
     application.ProcessMessages;
     ComputeProfit;
     First := Abs(Individual-First);
     Second := Abs(Individual-Second);

     if ((Gen mod G_Gap)=1) or (G_Gap=1) then
     begin
        assignfile(History,FilePath1 + '\Historyk.h');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,H[First+i]);
        closefile(History);

        assignfile(History,FilePath1 + '\Historyk.w');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,W[First+i]);
        closefile(History);

        assignfile(History,FilePath1 + '\Historyk.m');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,M[First+i]);
        closefile(History);

        assignfile(History,FilePath1 + '\Historyk.s');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,Sigma[First+i]);
        closefile(History);

        assignfile(History,FilePath1 + '\Historyk.v');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,Vijs[First+i]);
        closefile(History);

        assignfile(History,FilePath1 + '\Historyk.f');
        rewrite(History);
        for i := 1 to Individual do
           writeln(History,FileIndex[First+i]);
        closefile(History);
     end
     else
     begin
        if ((Gen mod G_Gap)=2) or ((G_Gap=2) and ((Gen mod G_Gap)=0)) then
        begin
           assignfile(History,FilePath1 + '\History1.h');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,H[First+i]);
           closefile(History);

           assignfile(History,FilePath1 + '\History1.w');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,W[First+i]);
           closefile(History);

           assignfile(History,FilePath1 + '\History1.m');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,M[First+i]);
           closefile(History);

           assignfile(History,FilePath1 + '\History1.s');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,Sigma[First+i]);
           closefile(History);

           assignfile(History,FilePath1 + '\History1.v');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,Vijs[First+i]);
           closefile(History);

           assignfile(History,FilePath1 + '\History1.f');
           rewrite(History);
           for i := 1 to Individual do
              writeln(History,FileIndex[First+i]);
           closefile(History);
        end;
     end;

     if ((Gen mod G_Gap)=1) or (G_Gap=1) then
     begin
        SucceedNo := 0; MartingaleNo:= 0; VisitGenNo := 0; VisitLibNo := 0;
//      if (Gen > WinSize + 11) then
//      begin
//         Reproduction(-1);
//         n := 34;
//      end
//      else
//         n := 1;

//      for i := n to Individual do
        for i := 1 to Individual do
        begin
           VisitGen := 0;  VisitLib := 0;
           Succeed := 0;   Martingale := 0;
           Value := Random(1000)/1000;
           if Value >= (FileIndex[First+i]/Individual) then  // Rank Selection
           begin
              q := (Wk[i]-Wk2[i])/Abs(Wk2[i]);  // Relative Fittness Selection
              p := 1 / (1+Exp(q));
              Value := Random(1000)/1000;
              if Value <= p then
                 Selection(i)
              else
              begin
                 ReadTree(First+i);
                 ComputeHi(i,Hi);
                 H[Second+i] := Hi;
                 WriteTree(Second+i);
                 FreeQueue(Queue);
                 root := Recover(Second+i,t1,t2);
                 DepthTable[i] := Depth2(root);
                 NodeTable[i] := Count_Node(root);
                 freetree(root);
              end;
           end
           else
              Selection(i);
           SucceedNo := SucceedNo + Succeed;
           MartingaleNo := MartingaleNo + Martingale;
           VisitGenNo := VisitGenNo + VisitGen;
           VisitLibNo := VisitLibNo + VisitLib;
        end;
        ResetWk;
     end
     else
     begin
        SucceedNo := 0; MartingaleNo:= 0; VisitGenNo := 0; VisitLibNo := 0;
        for i := 1 to Individual do
        begin
           ReadTree(First+i);
           ComputeHi(i,Hi);
           H[Second+i] := Hi;
           WriteTree(Second+i);
           FreeQueue(Queue);
           root := Recover(Second+i,t1,t2);
           DepthTable[i] := Depth2(root);
           NodeTable[i] := Count_Node(root);
           freetree(root);
           MartingaleNo := MartingaleNo + Martingale;
        end;
     end;

     ComputePstar;
     OutPut_msg('P(t+1) =  '+Floattostr(Pstar));
     ShowGraph(Gen,Pstar);
     application.ProcessMessages;
     for i := 1 to Individual do
       FileIndex[Second+i] := i;
     sort(1,Individual);
     StoreData;
     StoreBestTree;

     if (pfDiskFree < 100.0) or (pbCancel) then
     begin
        pbExit := true;
        Myini := TIniFile.Create(sInifile);
        Myini.WriteString('Common','Exit','1');
        Myini.WriteString('Common','No_Memory','1');
        Myini.Free;
     end;
Exit_Point:

   except
      pbExit := true;
      Myini := TIniFile.Create(sInifile);
      Myini.WriteString('Common','Running','0');
      Myini.WriteString('Common','Exit','1');
      Myini.Free;
      FreeQueue(Queue);
      closefile(out1);
      closefile(out2);
      application.Terminate;
      exit;
   end;
end;      (* end of while *)

   FreeQueue(Queue);
   if not pbExit then
   begin
      rewrite(Data);
      for i := 1 to piX_No do
          write(Data,X[i]:7:3,'  ');
      writeln(Data);
      for i := 1 to piY_No do
         write(Data,Y[i]:7:3,'  ');

     Myini := TIniFile.Create(sInifile);
     Myini.WriteString('Common','Stop_No','0');
     Myini.Free;
     closefile(Data);
   end
   else
      DumpData;
   closefile(out1);
   closefile(out2);
   Myini := TIniFile.Create(sInifile);
   Myini.WriteString('Common','Running','0');
   Myini.Free;
   application.Terminate;
   exit;
end;


procedure TForm1.TimerTimer(Sender:TObject);
begin                                     // 0 means checking current disk
    pfDiskFree := DiskFree(0) / 1000000;  // 3 means checking C disk
   StatusBar1.Panels[0].Text := 'Free Disk Space : ' + Floattostr(pfDiskFree) + 'MB';
   application.ProcessMessages;
end;


procedure TForm1.btnExitClick(Sender:TObject);
begin
   pbCancel := true;
end;


end.
