//  Santa Fe Institute Stock Market
//  Genetic Programming in Delphi, Tournament Selection

//  * Library Part *

unit Ver2A;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Matrix, IniFiles, ExtCtrls, ComCtrls, Math;

type
  TForm1 = class(TForm)
    edtMsg: TMemo;
    procedure FormActivate(Sender:TObject);
    procedure Output_msg(sMsg:string);
    procedure GetEnv(sIniFile:string);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
CONST
   Individual  = 500;
   FileAmount  = 1000;
   MaxSqrt     = 1.090748E+2466;
   XAmount     = 15;
   ExVarAmount = 15;
   Element_Amount = 75;
   WinSize     = 100;

type
   TwoPoints = ^TwoPointsNode;
   TwoPointsNode = record
      iData : byte;
      R     : real;
      Right : TwoPoints;
      Left  : TwoPoints;
   end;

   OnePoint = ^OnePointNode;
   OnePointNode = record
      iData: byte;
      R    : real;
      Next : OnePoint;
   end;
   STR4 = string[4];
   MatrixType = Array[1..WinSize+11] of real;

var
   piFunctionSet : byte;
   piBinarySet   : byte;
   piUnarySet    : byte;
   piTerminalSet : byte;
   piSetNo       : byte;
   piX_No,piY_No,piExVar_No : byte;
   psConst       : string[1];
   FilePath      : string[50];
   FileName      : string[50];
   DataFile      : string[50];
   TreeFile      : string[50];
   X, Y          : Array[1..XAmount] of real;
   ExVar         : Array[1..ExVarAmount] of real;
   ElementsTable : Array[1..Element_Amount] of Str4;
   Mape          : Array[1..Individual] of real;
   FileIndex_L   : Array[1..FileAmount] of integer;
   Disk, Data    : TextFile;
   Gen, GNumber  : integer;
   Queue, This   : OnePoint;
   fault, pbExit : boolean;
   First,Second  : integer;
   DimA,DimB,DimC: MatrixType;
   Out_Count     : integer;
   Backup,MapeFile,ExData : Textfile;
   piDoDim       : byte;
   sIniFile,psWaitPath,ExDataFile : string;
   Full          : integer;
   Grow          : integer;
   Reproduct     : integer;
   Mutation      : integer;
   MutProb       : real;
   DepthLimit    : integer;
   LeaveProb     : integer;
   MaxExp        : real;
   MutMode       : byte;
   G_Gap         : byte;
   Theta1        : real;
   Theta2        : real;
   TryNo         : byte;


function ReadINI(sSecName,sAppName,sIniFile:string):string;
var
    Myini : TIniFile;
begin
   Myini := TIniFile.Create(sInifile);
   result := Myini.ReadString(sSecName,sAppName,'Not_Found');
   Myini.Free;
   if result = 'Not_Found' then
      result := '';
end;


procedure TForm1.GetEnv(sIniFile:string);
var
   iPos,i : integer;
   sSet : string;
begin
   Full := strtoint(ReadINI('Business_School','Full',sIniFile));
   Grow := strtoint(ReadINI('Business_School','Grow',sIniFile));
   Reproduct := strtoint(ReadINI('Business_School','Reproduct',sIniFile));
   Mutation := strtoint(ReadINI('Business_School','Mutation',sIniFile));
   MutProb := strtofloat(ReadINI('Business_School','MutProb',sIniFile));
   DepthLimit := strtoint(ReadINI('Business_School','DepthLimit',sIniFile));
   LeaveProb := strtoint(ReadINI('Business_School','LeaveProb',sIniFile));
   MaxExp := strtofloat(ReadINI('Business_School','MaxExp',sIniFile));
   MutMode := strtoint(ReadINI('Business_School','MutMode',sIniFile));
   G_Gap := strtoInt(ReadINI('Business_School','G_Gap',sIniFile));
   Theta1 := strtofloat(ReadINI('Business_School','Theta1',sIniFile));
   Theta2 := strtofloat(ReadINI('Business_School','Theta2',sIniFile));
   TryNo := strtoint(ReadINI('Business_School','TryNo',sIniFile));

   sSet := uppercase(trim(ReadINI('Common','BinarySet',sIniFile)));
   i := 1;
   piBinarySet := 0;
   piSetNo := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piBinarySet := piBinarySet + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','UnarySet',sIniFile)));
   piUnarySet := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piUnarySet := piUnarySet + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;
   piFunctionSet := piUnarySet + piBinarySet;

   sSet := uppercase(trim(ReadINI('Common','X_Var',sIniFile)));
   piX_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piX_No := piX_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','Y_Var',sIniFile)));
   piY_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piY_No := piY_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   sSet := uppercase(trim(ReadINI('Common','Extra_Var',sIniFile)));
   piExVar_No := 0;
   while (sSet <> '') do
   begin
      iPos := Pos('\',sSet);
      if iPos > 0 then
      begin
         ElementsTable[i] := trim(Copy(sSet,1,iPos-1));
         sSet := Copy(sSet,iPos+1,length(sSet));
         i := i + 1;
         piExVar_No := piExVar_No + 1;
         piSetNo := piSetNo + 1;
      end
      else
         sSet := '';
   end;

   piTerminalSet := piX_No + piY_No + piExVar_No;
   psConst := uppercase(trim(ReadINI('Common','Const_Item',sIniFile)));
   if psConst <> '' then
   begin
      ElementsTable[i] := psConst;
      piSetNo := piSetNo + 1;
      piTerminalSet := piTerminalSet + 1;
   end;
end;


function ApPath:string;
begin
   result := ExtractFilePath(application.ExEName);
end;


function Obtain(str:str4):byte;
var
   i : byte;
begin
   Str := trim(Str);
   result := 0;
   for i := 1 to piSetNo do
   begin
      if ElementsTable[i] = Str then
      begin
         result := i;
         exit;
      end;
   end;
end;


function Depth2(T:TwoPoints):integer;
var
    l, r : integer;
begin
   result := 0;
   if (T <> nil) then
   begin
      l := Depth2(T^.Left);
      r := Depth2(T^.Right);
      if l >= r then
         result := l + 1
      else
         result := r + 1;
   end
end;   (* end of Depth *)


function Count_Node(T:TwoPoints):integer;
var
    l, r : integer;
begin
   result := 0;
   if (T <> nil) then
   begin
      l := Count_Node(T^.Left);
      r := Count_Node(T^.Right);
      result := l + r + 1;
   end
end;   (* end of Depth *)


procedure OpenFile(Mode:integer;G:integer);
var
   l : byte;
   Temp : string;
   i    : integer;
begin
   Temp := inttostr(G);
   l := Length(Temp);
   for i := l + 1 to 4 do
      Temp := '0' + Temp;

   case G div 100 of
   0: FileName := FilePath +'0\'+ FileName + Temp + '.dat';
   1: FileName := FilePath +'1\'+ FileName + Temp + '.dat';
   2: FileName := FilePath +'2\'+ FileName + Temp + '.dat';
   3: FileName := FilePath +'3\'+ FileName + Temp + '.dat';
   4: FileName := FilePath +'4\'+ FileName + Temp + '.dat';
   5: FileName := FilePath +'5\'+ FileName + Temp + '.dat';
   6: FileName := FilePath +'6\'+ FileName + Temp + '.dat';
   7: FileName := FilePath +'7\'+ FileName + Temp + '.dat';
   8: FileName := FilePath +'8\'+ FileName + Temp + '.dat';
9,10: FileName := FilePath +'9\'+ FileName + Temp + '.dat';
   end;

   try
      assignfile(Disk,FileName);
      case Mode of
      0: Reset(Disk);
      1: Rewrite(Disk);
      end;
   except
      MessageDlg('Ver2A: File('+inttostr(Gen)+') Open Fail, Mode='+inttostr(Mode)+'File Name='+FileName,mtError,[],0);
   end;
end;  (* end of assignfile *)


procedure PostOrder(var t:TwoPoints);
var
    ch : str4;
begin
   if t <> nil then
   begin
      PostOrder(t^.Left);
      PostOrder(t^.Right);
      ch := trim(ElementsTable[t^.iData]);
      write(Disk,ch);
      if t^.iData = piSetNo then
         write(Disk,'(',t^.R:4:2,' )');
      t := nil;
   end;
end;


procedure FreeTree(var t:TwoPoints);
begin
   try
      if t = nil then
         exit;

      if t.Left <> nil then
         FreeTree(t.Left);

      if t.Right <> nil then
          FreeTree(t.Left);

      Dispose(t);
      t := nil;
   except
      t := nil;
   end;
end;


procedure FreeQueue(var q:OnePoint);
var ip : OnePoint;
begin
  try
     if q <> nil then
        while q <> nil do
        begin
          ip := q;
          q := q^.next;
          dispose(ip);
        end;
      q := nil;
   except
      q := nil;
   end;
end;


procedure StoreTree(var t:TwoPoints;i:integer);
begin
   FileName := 'T';
   OpenFile(1,i);
   PostOrder(t);
   t := nil;
   FileIndex_L[i] := i;
   CloseFile(Disk);
end;    (* end of StoreTree *)


function ReadSTR4:str4;
var
    str      : str4;
    ch1, ch2 : char;
begin
    Str := '';
    read(Disk,ch1);
    Str := ch1;
    case ch1 of
    'E','L','C','A',
    'e','l','c','a'
                   : begin
                     read(Disk,ch1,ch2);
                     Str := Concat(Str,ch1,ch2);
                     end;
    'x','X','Y','y','z','Z'
                   : begin
                     read(Disk,ch1,ch2);
                     Str := Concat(Str,ch1,ch2);
                     end;
    'S' ,'s'       : begin
                     read(Disk,ch1);
                     case ch1 of
                     'q','Q'
                            : begin
                              Str := Concat(Str,ch1);
                              read(Disk,ch1,ch2);
                              Str := Concat(Str,ch1,ch2);
                              end;
                     'i','I'
                            : begin
                              read(Disk,ch2);
                              Str := Concat(Str,ch1,ch2);
                              end;
                     end;
                 end;
    end;
    result := Str;
end;


procedure ReadTree(i:integer);
var
    q   : OnePoint;
    ch  : char;
    str : str4;
begin
   if i = -1 then
   begin
      assignfile(Disk,TreeFile);
      reset(Disk);
   end
   else
   begin
      FileName := 'T';
      OpenFile(0,i);
   end;
   New(Queue);
   str := ReadSTR4;
   Queue^.iData := Obtain(str);
   if Queue^.iData = piSetNo then
      read(Disk,ch,Queue^.R,ch,ch);
   queue^.next := nil;
   this := queue;
   while not eof(Disk) do
   begin
      New(q);
      str := ReadSTR4;
      q^.iData := obtain(str);
      q^.next := nil;
      if q^.iData = piSetNo then
         read(Disk,ch,q^.R,ch,ch);
      this^.next := q;
      this := this^.next;
   end;  (* end of while not eof *)
   this := queue;
   CloseFile(Disk);
end;    (* end of ReadTree *)


function GetOp(var empty:boolean;var Value:real):byte;
begin
   if This <> nil then
   begin
      result := This^.iData;
      Value := This^.R;
      This  := This^.Next;
   end
   else
       empty := true;
end;


function Compute(op:byte;value1,value2:extended):extended;
var
   sOp : str4;
begin
   sOp := ElementsTable[Op];
   case sOp[1] of
      '+' : Compute := value2 + value1;
      '-' : Compute := value2 - value1;
      '*' : Compute := value2 * value1;
      '%' : if value1 = 0.0 then
               Compute := 1
            else
               Compute := value2 / value1;
      'E' : begin
             if Abs(Value1) > MaxExp then
                Compute := Exp(MaxExp)
             else
                Compute := Exp(value1);
          end;
      'L': if value1 = 0.0 then
              Compute := 0.0
           else
              Compute := Ln(Abs(value1));
      'S' : begin
               case sOp[2] of
               'Q' : Compute := sqrt(abs(value1));
               'I' : Compute := Sin(value1);
               end;
            end;
    'C' : Compute := Cos(value1);
    'A' : Compute := Abs(value1);
   end;
end;  (* End of Compute *)


function ComputeVij(u:integer):extended;
type
     spoint = ^StackNode;
     StackNode = record
        Value : extended;
        next  : spoint;
     end;
var
    op    : byte;
    Value : real;
    Top   : spoint;
    empty : boolean;
    Order : integer;
    Vij   : extended;

   procedure FreeStack(var s:spoint);
   var ip : spoint;
   begin
Dispose(s);
exit;
      try
         while s <> nil do
         begin
            ip := s;
            s := s^.next;
            dispose(ip);
         end;
         s := nil;
      except
         s := nil;
      end;
   end;

   procedure push(value:extended);
   var
       p : spoint;
   begin
      New(p);
      p^.value := value;
      p^.next := Top;
      Top := p;
   end;

   function pop:extended;
   var
        p : spoint;
   begin
      if Top <> nil then
      begin
         p := Top;
         pop := Top^.value;
         Top := Top^.next;
         Dispose(p);
      end
      else
      begin
         MessageDlg('Error: Stack underfollow. ',mtWarning,[mbOK],0);
         exit;
      end
   end;

begin
   try
      This := Queue;
      Top := nil;   (* Initial Stack while compute another record of Data *)
      empty := false;
      op := GetOp(empty,Value);
      while not empty do   (* Do while queue isn't empty *)
      begin
         if (op > piFunctionSet) then
         begin
            if op = piSetNo then
               Push(Value)
            else
            begin
               op := op - piFunctionSet;
               if op <= piX_No then
                  Push(DimB[u-1-op])
               else
               begin
                  op := op - piX_No;
                  if op <= piY_No then
                     Push(DimC[u-1-op])
                  else
                  begin
                     op := op - piY_No;
                     if op <= piExVar_No then
                        Push(Exvar[op])
                     else
                     begin
                        MessageDlg('Incorrect operator !',mtWarning,[mbOK],0);
                        exit;
                     end;
                  end;
               end;
            end;
         end
         else
         begin
            if (op <= piBinarySet) then
               push(compute(op,pop,pop))
                                          (* Note: The compute function *)
            else   (* op is Unary *)            (* is second value "op" *)
               push(compute(op,pop,-1));  (* first value *)
         end;
         op := GetOp(empty, Value);
      end;    (* end of while not empty *)

      if Top^.next <> nil then
      begin
         MessageDlg('Error: Data compute not match in T' ,mtWarning,[mbOK],0);
         exit;
      end;

      Vij := Top^.Value;

      if Abs(Vij) <= MaxExp then
         Vij := (Exp(Theta2*Vij)-Exp(-Theta2*Vij))/(Exp(Theta2*Vij)+Exp(-Theta2*Vij))
      else
      begin
        if Vij > 0.0 then
           Vij := 1.0
        else
           Vij := -1.0;
      end;
//    Vij := DimC[u-1] * (1 + Theta1 * Vij);
      Vij := (DimB[u-1]+10.0) * (1 + Theta1 * Vij);
      result := Vij;

      FreeStack(Top);
     except
        FreeStack(Top);
     end;
end;


function ComputeMape:extended;
var
   u : integer;
   Vij2 : extended;
begin
   result := 0.0;
   fault := false;
   for u := 111 downto 102 do
   begin
      Vij2 := ComputeVij(u);
      if fault then
         break;
      result := result + Abs((Vij2-DimA[u])/DimA[u]);
   end;
end;


procedure MakeFullTree(var t:TwoPoints;d,depth:integer);
var
    p        : TwoPoints;
    i,j,sign : byte;
    val      : real;
begin
   if d < depth then         (* Get element from function set *)
   begin
       New(p);
       p^.R := 111.0;
//     for j := 1 to Random(256) do
//         i := Random(j);
       P^.iData := Random(piFunctionSet) + 1;
       MakeFullTree(p^.left,d+1,depth);
       if (p^.iData <= piBinarySet) then        (* This element is binary *)
          MakeFullTree(p^.right,d+1,depth)
       else
          p^.right := nil;                 (* This element is unary *)
   end
   else
      if d = depth then       (* Get element from terminal set *)
      begin
         New(p);
         p^.R := 111.0;
         p^.iData := Random(piTerminalSet) + piFunctionSet + 1;
         if p^.iData = piSetNo then           (* Choice Const R *)
         begin
            val := Random(10)+Random(100)/100;
            sign := Random(2);
            if sign = 1 then
              val := val*(-1);
            p^.R := val;
         end;
         P^.right := nil;
         p^.left := nil;
      end;
   t := p;
end;    (* end of MakeFullTree *)


procedure MakeGrowTree(var t:TwoPoints;d,depth:integer);
var
    p        : TwoPoints;
    i,j,sign : byte;
    val      : real;
begin
   if d <= depth then
   begin
      if d < depth then     (* d < depth, so get element from all element *)
      begin
//       for j := 1 to Random(256) do
//           i := Random(j);
         i := Random(2);
         if i = 0 then
            i := Random(piFunctionSet) + 1
         else
            i := Random(piTerminalSet) + piFunctionSet + 1;
      end
      else                  (* d = depth, so get element from terminal set *)
      begin
//       for j := 1 to Random(256) do
//           i := Random(j);
         i := Random(piTerminalSet) + piFunctionSet + 1;
      end;
      New(p);
      p^.R := 111.0;
      if (i > piSetNo) or (i <= 0) then
      begin
         MessageDlg('Data Generate Error !!  It order is '+inttostr(i),mtWarning,[],0);
         exit;
      end;
      P^.iData := i;
      if (p^.iData <= piFunctionSet) then
      begin
         MakeGrowTree(p^.left,d+1,depth);
         if (p^.iData <= piBinarySet) then        (* This element is binary *)
            MakeGrowTree(p^.right,d+1,depth)
         else
            p^.right := nil;                 (* This element is unary *)
      end
      else                                   (* This element is terminal *)
      begin
         if i = piSetNo then
         begin
            val := Random(10)+Random(100)/100;
            sign := Random(2);
            if sign = 1 then
               val := val*(-1);
            p^.R := val;
         end;
         P^.right := nil;
         p^.left  := nil;
      end;
      t := p;
   end;    (* end of d <= depth *)
end;    (* end of MakeGrowTree *)


procedure Generate(N:integer);
var
    j, depth : integer;
    t        : TwoPoints;
begin    (* Generate *)
   j := (Individual)*N+1;
   for depth := 2 to 6 do
   begin
      n := 1;
      repeat                 (* Generated tree by full method *)
         MakeFullTree(t,1,depth);
         StoreTree(t,j);
         FreeTree(t);
         ReadTree(j);         (* Read file of tree i and build a queue list *)
         Mape[j] := ComputeMape;
         n := n + 1;
         j := j + 1;
         FreeQueue(Queue);      (* Return all the nodes of queue of tree i *)
      until n > Full;
      n := 1;
      repeat
         MakeGrowTree(t,1,depth);
         StoreTree(t,j);
         FreeTree(t);
         ReadTree(j);         (* Read file of tree i and build a queue list *)
         Mape[j] := ComputeMape;
         n := n + 1;
         j := j + 1;
         FreeQueue(Queue);     (* Return all the nodes of queue of tree i *)
      until n > Grow;
   end;    (* end of for loop depth *)
end;   (* end of generate *)


function Recover(i:integer;var Atoms,Leaves:integer):TwoPoints;
type
    StackPoint = ^StackNode;
    StackNode = record
        Tree : TwoPoints;
        Next : StackPoint;
    end;
var
    top : StackPoint;
    op  : byte;
     p  : TwoPoints;
    str : str4;
    ch  : char;

   procedure push(p:TwoPoints);
   var
       q : StackPoint;
   begin
      New(q);
      q^.Tree := p;
      q^.Next := Top;
      Top := q;
   end;

   function pop:TwoPoints;
   var
       q : StackPoint;
   begin
      if top <> nil then
      begin
         pop := top^.Tree;
         q := top;
         top := top^.Next;
         dispose(q);
      end
      else
         MessageDlg('Stack underfollow !! ',mtWarning,[],0);
   end;

begin
    top := nil;
    Atoms := 0;    Leaves := 0;
    FileName := 'T';
    if i >= 0 then
       OpenFile(0,i)
    else
    begin
       i := abs(i);
       OpenFile(-2,i);
    end;

    while not eof(Disk) do
    begin
       str := ReadSTR4;
       op := obtain(str);
       Atoms := Atoms + 1;
       if (op > piFunctionSet) then
          Leaves := Leaves + 1;
       New(p);
       p^.iData := op;
       if (op > piFunctionSet) then
       begin
          if P^.iData = piSetNo then
             read(Disk,ch,P^.R,ch,ch);
          p^.Left := nil;
          p^.Right := nil;
          push(p);
       end
       else
          if (op <= piBinarySet) then
          begin
             p^.right := pop;
             p^.left := pop;
             push(p);
          end
          else
          begin      (* op is unary *)
             p^.left := pop;
             P^.right := nil;
             push(p);
          end;
    end;   (* end of while not eof  *)

    recover := top^.Tree;
    closefile(Disk);
end;   (* end of recover *)


procedure WriteTree(i:integer);
begin
   FileName := 'T';
   OpenFile(1,i);
   This := Queue;
   while This <> nil do
   begin
      write(Disk,ElementsTable[This^.iData]);
      if This^.iData = piSetNo then
         write(Disk,'(',This^.R:4:2,' )');
      This := This^.Next;
   end;
   FileIndex_L[i] := i;
   closefile(Disk);
end;


(* Return the file index of tree *)
function ChoiceTree:integer;
var
    i, j : integer;
begin
   i := Random(Individual) + 1;
   j := Random(Individual) + 1;

   if Mape[i] <= Mape[j] then
       ChoiceTree := FileIndex_L[First+i]
   else
       ChoiceTree := FileIndex_L[First+j];
end;


procedure Reproduction(n:integer);
var
    Choice : integer;
begin
   Choice := ChoiceTree;
   ReadTree(Choice);
   WriteTree(Second+n);
   Mape[n] := ComputeMape;
   FreeQueue(Queue);
end;   (* end of Reproduction *)


procedure Mutate0(i:integer);
var
    n, Ran : integer;
begin
   n := ChoiceTree;
   ReadTree(n);
   This := Queue;
  while This <> nil do
   begin
      Ran := Random(1000)+1;
      if Ran <= MutProb then
      begin
         if (This^.iData <= piBinarySet) then
         begin
            This^.iData := Random(piBinarySet) + 1;
         end
         else
            if (This^.iData > piBinarySet) and (This^.iData <=piFunctionSet) then
            begin
               This^.iData := Random(piUnarySet) + piBinarySet + 1;
            end
            else  (* In TerminalSet  *)
            begin
               This^.iData := Random(piTerminalSet) + piFunctionSet + 1;
               if This^.iData = piSetNo then
               begin
                  This^.R := Random(10)+Random(100)/100;
                  Ran := Random(2);
                  if Ran = 1 then
                     This^.R := (-1) * This^.R;
               end
               else
                  This^.R := 111.0;
            end;     (* In Terminal Set *)
      end;  (* if Ran <= MutProb *)
      This := This^.Next;
   end;   (* while This <> nil *)  while This <> nil do
   begin
      Ran := Random(1000)+1;
      if Ran <= MutProb then
      begin
         if (This^.iData <= piBinarySet) then
         begin
            This^.iData := Random(piBinarySet) + 1;
         end
         else
            if (This^.iData > piBinarySet) and (This^.iData <=piFunctionSet) then
            begin
               This^.iData := Random(piUnarySet) + piBinarySet + 1;
            end
            else  (* In TerminalSet  *)
            begin
               This^.iData := Random(piTerminalSet) + piFunctionSet + 1;
               if This^.iData = piSetNo then
               begin
                  This^.R := Random(10)+Random(100)/100;
                  Ran := Random(2);
                  if Ran = 1 then
                     This^.R := (-1) * This^.R;
               end
               else
                  This^.R := 111.0;
            end;     (* In Terminal Set *)
      end;  (* if Ran <= MutProb *)
      This := This^.Next;
   end;   (* while This <> nil *)  while This <> nil do
   begin
      Ran := Random(1000)+1;
      if Ran <= MutProb then
      begin
         if (This^.iData <= piBinarySet) then
         begin
            This^.iData := Random(piBinarySet) + 1;
         end
         else
            if (This^.iData > piBinarySet) and (This^.iData <=piFunctionSet) then
            begin
               This^.iData := Random(piUnarySet) + piBinarySet + 1;
            end
            else  (* In TerminalSet  *)
            begin
               This^.iData := Random(piTerminalSet) + piFunctionSet + 1;
               if This^.iData = piSetNo then
               begin
                  This^.R := Random(10)+Random(100)/100;
                  Ran := Random(2);
                  if Ran = 1 then
                     This^.R := (-1) * This^.R;
               end
               else
                  This^.R := 111.0;
            end;     (* In Terminal Set *)
      end;  (* if Ran <= MutProb *)
      This := This^.Next;
   end;   (* while This <> nil *)
   WriteTree(Second+i);
   FreeQueue(Queue);
end;    (*  end of Mutate *)


function Depth(T:TwoPoints;n:integer):integer;
var
    l, r : integer;
begin
   if (T <> nil) and (n < DepthLimit+2) then
   begin
      n := n + 1;
      l := Depth(T^.Left,n);
      r := Depth(T^.Right,n);
      if l >= r then
         Depth := l
      else
         Depth := r;
   end
   else
      Depth := n;
end;   (* end Of Depth *)


procedure CutTree(var Count:integer;var T,Here:TwoPoints;var Turn:integer);
begin
   if (Count > 1) and (T <> nil) then
   begin
      Here := T;   Turn := 0;
      Count := Count - 1;
      CutTree(Count,T^.Left,Here,Turn);
      if (Count = 1) and (Turn = 0) then
         if Here^.Left <> nil then
            Count := 0;
      if (Count >= 1) and (T^.Right <> nil) then
      begin
         Turn := 1;
         if Count > 1 then
           CutTree(Count,T^.Right,Here,Turn)
         else
         begin
            Here := T;
            Count := 0;
         end;
      end;
   end;
end;   (* end of CutTree *)


function Cut(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
   Count := Random(Atoms)+1;
   if Count = 1 then
   begin
      Here := T;
      Turn := 2;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      CutTree(Count,T,Here,Turn);
   end;
   Cut := Here;
end;


procedure Mutate1(i:integer);
var
    n, Ran, d : integer;
    Atoms, Turn,Leaves : integer;
    t, s, Here, Tree : TwoPoints;
begin
   n := ChoiceTree;
   Ran := Random(1000)+1;

   if Ran <= MutProb then
   begin
      MakeGrowTree(t,1,6);
      Tree := Recover(n,Atoms,Leaves);
      Here := Cut(Tree,Atoms,Turn);
      case turn of
      0 : begin
             s := Here^.left;
             Here^.left := t;
             FreeTree(s);
          end;
      1 : begin
             s := Here^.Right;
             Here^.Right := t;
             FreeTree(s);
          end;
      2 : begin
             FreeTree(Tree);
             Tree := t;
          end;
      end;
      d := Depth(Tree,0);

      if d > DepthLimit then
      begin
         FreeTree(Tree);
         ReadTree(n);
         WriteTree(Second+i);
         FreeQueue(Queue);
      end
      else
      begin
         StoreTree(Tree,Second+i);
         FreeTree(Tree);
      end
   end
   else
   begin
      ReadTree(n);  (* Read file of tree i and build a queue list *)
      WriteTree(Second+i);
      FreeQueue(Queue);
   end;
end;    (*  end of Mutate1 *)


procedure Preorder(var Count:integer;T:TwoPoints;var Here:TwoPoints;var Turn:integer);
begin
   Here := T;
   if T <> nil then
      if (Count > 1) and (T^.iData <= piFunctionSet) then
      begin
         Here := T;   Turn := 0;
         Count := Count - 1;
         Preorder(Count,T^.Left,Here,Turn);
         if (Count = 1) and (Turn = 0) then
            if Here^.Left <> nil then
               if Here^.Left^.iData <= piFunctionSet then
                  Count := 0;
         if t^.Right <> nil then
            if (Count >= 1) and (T^.Right^.iData <= piFunctionSet) then
            begin
               Turn := 1;
               if Count > 1 then
                  Preorder(Count,T^.Right,Here,Turn)
               else
               begin
                  Here := T;
                  Count := 0;
               end;
            end;
      end;
end;   (* end of Preorder *)


procedure GetLeave(var Count:integer;T:TwoPoints;var Here:TwoPoints;var Turn:integer);
begin
   Here := nil;
   if T <> nil then
      if (T^.iData <= piFunctionSet) and (Count > 0) then
      begin
         if T^.Left <> nil then
            if T^.Left^.iData > piFunctionSet then
            begin
               Count := Count - 1;
               if Count = 0 then
               begin
                 Turn := 0;
                 Here := T;
               end;
            end
            else
               GetLeave(Count,T^.Left,Here,Turn);

         if Count > 0 then
            if T^.Right <> nil then
               if T^.Right^.iData > piFunctionSet then
               begin
                  Count := Count - 1;
                  if Count = 0 then
                  begin
                     Turn := 1;
                     Here := T;
                  end;
               end
               else
                  GetLeave(Count,T^.Right,Here,Turn);
      end;
end;


function CutLeave(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
 Count := Random(Atoms)+1;

   if T^.iData > piFunctionSet then
   begin
      Here := T;
      Turn := -1;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      GetLeave(Count,T,Here,Turn);
   end;
   CutLeave := Here;
end;


function CutPoint(T:TwoPoints;Atoms:integer;var Turn:integer):TwoPoints;
var
    Count : integer;
    Here  : TwoPoints;
begin
    Count := Random(Atoms)+1;

   if Count = 1 then
   begin
      Here := T;
      Turn := -1;         (* Here = Root of Tree *)
   end
   else
   begin
      Here := nil; Turn := 0;
      Preorder(Count,T,Here,Turn);
   end;
   CutPoint := Here;
end;


procedure Crossover(i:integer);
var
    Tree1, Tree2, q   : TwoPoints;
    Here1, Here2      : TwoPoints;
    k, Atoms1, Atoms2 : integer;
    Leaves1, Leaves2  : integer;
    Turn1,Turn2,Mode  : integer;
    n1, n2, d1, d2    : integer;
begin
   n1 := ChoiceTree;
   Tree1 := Recover(n1,Atoms1,Leaves1);
   n2 := ChoiceTree;
   Tree2 := Recover(n2,Atoms2,Leaves2);
   Mode  := Random(10);
   if Mode < LeaveProb then
      Here1 := CutLeave(Tree1,Leaves1,Turn1)   (* Choice Leaves *)
   else
      Here1 := CutPoint(Tree1,Atoms1-Leaves1,Turn1);
   Mode := Random(10);
   if Mode < LeaveProb then
      Here2 := CutLeave(Tree2,Leaves2,Turn2)
   else
      Here2 := CutPoint(Tree2,Atoms2-Leaves2,Turn2);
                           (* Here1 or Here2  points to root of tree. *)
   if Turn1 = -1 then      (* So, We make a dummy head to  point it.  *)
   begin                   (* This dummy head's Data is "Plus", and   *)
      New(Tree1);          (* Right is nil, but it's Left points to   *)
      Tree1^.iData := 1;   (* Here1 or Here2.                         *)
      Tree1^.Right := nil;
      Tree1^.Left := Here1;
      Here1 := Tree1;
      Turn1 := 0;
   end;
   if Turn2 = -1 then
   begin
      New(Tree2);
      Tree2^.iData := 1;
      Tree2^.Right := nil;
      Tree2^.Left := Here2;
      Here2 := Tree2;
      Turn2 := 0;
   end;
   if Turn1 = 0 then
   begin
      q := Here1^.Left;
      if Turn2 = 0 then                (* Turn1=0 and Turn2=0 *)
      begin
         Here1^.Left := Here2^.Left;
         Here2^.Left := q;
      end
      else                              (* Turn1=0 and Turn2=1 *)
      begin
         Here1^.Left  := Here2^.Right;
         Here2^.Right := q;
      end
   end
   else    (* Turn1 = 1  *)
   begin
      q := Here1^.Right;
      if Turn2 = 0 then                   (* Turn1=1 and Turn2=0 *)
      begin
         Here1^.Right := Here2^.Left;
         Here2^.Left  := q;
      end
      else                                 (* Turn1=1 and Turn2=1 *)
      begin
         Here1^.Right := Here2^.Right;
         Here1^.Right := q;
      end;
   end;
   if (Tree1^.iData=1) and (Tree1^.Right = nil) then
      Tree1 := Tree1^.Left;
   if (Tree2^.iData=1) and (Tree2^.Right = nil) then
      Tree2 := Tree2^.Left;

   d1 := Depth(Tree1,0);   d2 := Depth(Tree2,0);
   k := 0;
   if d1 <= DepthLimit then
   begin
      StoreTree(Tree1,Second+i);
      FreeTree(Tree1);
   end
   else
      if (d2 <= DepthLimit) then
      begin
         StoreTree(Tree2,Second+i);
         FreeTree(Tree2);
      end
      else
      begin
         ReadTree(n1);
         WriteTree(Second+i);
         FreeQueue(Queue);
      end;
end;


procedure TForm1.OutPut_Msg(sMsg:string);
begin
   if Out_Count < 200 then
      edtMsg.lines.add(inttostr(Out_Count) + ': '+sMsg)
   else
      edtMsg.Lines[Out_Count Mod 200] := inttostr(Out_Count) + ': '+sMsg;
   Out_Count := Out_Count + 1;
   application.ProcessMessages;
end;


procedure DumpData;
var
    i : integer;
    Myini : TIniFile;
begin
   Myini := TIniFile.Create(sInifile);
   Myini.WriteString('Common','Stop_No',inttostr(Gen));
   Myini.Free;
   if fileExists(psWaitPath+'Backup2.txt') then
   begin
      deletefile(psWaitPath+'Backup22.txt');
      application.ProcessMessages;
      renameFile(psWaitPath+'Backup2.txt',psWaitPath+'Backup22.txt');
      application.ProcessMessages;
   end;

   assignfile(Backup,psWaitPath+'Backup2.txt');
   rewrite(Backup);
   writeln(Backup,'GEN=',inttostr(Gen),'  First='+inttostr(First),'  Second='+inttostr(Second));

   for i := 0 to XAmount do
       writeln(Backup,X[i]:12:5);

   for i := 0 to XAmount do
       writeln(Backup,Y[i]:12:5);

   for i := 1 to FileAmount do
       writeln(backup,FileIndex_L[i]);

   writeln(Backup,First,' ',Second);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimA[i]);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimB[i]);

   for i := 1 to WinSize+11 do
       writeln(Backup,DimC[i]);

   for i := 1 to Individual do
       writeln(Backup,Mape[i]);

   closefile(Backup);
end;


procedure ReadOldData;
var i : integer;
    sItem : string;
begin
   if piExVar_No > 0 then
   begin
      assignfile(ExData,ExDataFile);
      reset(ExData);
      for i := 1 to Gen - 1 do
      begin
         readln(ExData);
         if eof(ExData) then
         begin
            closefile(ExData);
            reset(ExData);
            for i := 1 to piExVar_No do
               read(ExData,ExVar[i]);
            readln(ExData);
         end;
      end;

      for i := 1 to piExVar_No do
          read(ExData,ExVar[i]);
      readln(ExData);
      if eof(ExData) then
      begin
         closefile(ExData);
         Reset(ExData);
      end;
   end;
   assignfile(Backup,psWaitPath+'Backup2.txt');
   reSet(Backup);
   readln(Backup,sItem);
   for i := 0 to XAmount do
       readln(Backup,X[i]);

   for i := 0 to XAmount do
       readln(Backup,Y[i]);

   for i := 1 to FileAmount do
       readln(Backup,FileIndex_L[i]);

   readln(Backup,First,Second);

   for i := 1 to WinSize+11 do
       readln(Backup,DimA[i]);

   for i := 1 to WinSize+11 do
       readln(Backup,DimB[i]);

   for i := 1 to WinSize+11 do
       readln(Backup,DimC[i]);

   for i := 1 to Individual do
       readln(Backup,Mape[i]);

    closefile(Backup);
end;


procedure Wait(n:integer);
label try_again;
var
   i : integer;
begin
   try
      while (not FileExists(psWaitPath+'Gen' + inttostr(n))) and (not pbExit) do
      begin
          for i := 1 to 10000 do
             application.ProcessMessages;
          if ReadINI('Common','Exit',ApPath + 'Ver2.ini') = '1' then
          begin
            pbExit := true;
            exit;
          end;
      end;
  except
     pbExit := true;
  end;
end;


procedure TForm1.FormActivate(Sender:TObject);
label Exit_Point;
var
    Value : real;
    i,iTry,u : integer;
    Vij2,ESum1,ESum2 : extended;
    Myini : TIniFile;
    DimAFile,DimBFile,DimCFile : TextFile;
begin
   sIniFile := ApPath + 'Ver2A.ini';
   GetEnv(sIniFile);
   Myini := TIniFile.Create(sInifile);
   Myini.WriteString('Common','Running','1');
   Myini.Free;
   Myini := TIniFile.Create(ApPath + 'Ver2.ini');
   Myini.WriteString('Common','Exit','0');
   Myini.WriteString('Common','No_Memory','0');
   Myini.Free;
   pbExit := false;
   Randomize;

   if not FileExists(sIniFile) then
   begin
      MessageDlg(sIniFile + ' Not Found!',mtError,[mbOK],0);
      application.Terminate;
      exit;
   end;
   DataFile := ReadINI('Common','Data_File',sIniFile);
   ExDataFile := ReadINI('Common','ExData_File',sIniFile);
   FilePath := ReadINI('Common','Tree_Path',sIniFile);
   GNumber := strtoint(ReadINI('Business_School','Gen_No',sIniFile));
   GEN := strtoint(ReadINI('Common','Stop_No',sIniFile));
   piDoDim := strtoint(ReadINI('Common','Do_Dim',sIniFile));
   psWaitPath := ReadINI('Common','Data_Path',sIniFile);

   if GEN = 0 then
   begin
      if piExVar_No > 0 then
      begin
         assignfile(ExData,ExDataFile);
         reset(ExData);
      end;   
      assignfile(Data,DataFile);
      reset(Data);
      for i := 0 to XAmount do
         read(Data,X[i]);
      readln(Data);
      for i := 0 to XAmount do
         read(Data,Y[i]);
      CloseFile(Data);

      if piDoDim = 0 then
      begin
         assignfile(DimAFile,ReadINI('Common','DimA_File',sIniFile));
         assignfile(DimBFile,ReadINI('Common','DimB_File',sIniFile));
         assignfile(DimCFile,ReadINI('Common','DimC_File',sIniFile));
         reset(DimAFile);
         reset(DimBFile);
         reset(DimCFile);
         for i := 1 to WinSize + 11 do
         begin
            readln(DimAFile,DimA[i]);
            readln(DimBFile,DimB[i]);
            readln(DimCFile,DimC[i]);
         end;
         closefile(DimAFile);
         closefile(DimBFile);
         closefile(DimCFile);
      end;
      Gen := 1;
      First := 500;
      Second := 0;
      edtMsg.Lines[0] := 'Processing Generation        1 ';
      Out_Count := 1;
      application.ProcessMessages;
      Generate(0);
      application.ProcessMessages;
   end
   else
   begin
      ReadOldData;
      assignfile(Data,DataFile);
      if piDoDim = 0 then
      begin
         assignfile(DimAFile,ReadINI('Common','DimA_File',sIniFile));
         assignfile(DimBFile,ReadINI('Common','DimB_File',sIniFile));
         assignfile(DimCFile,ReadINI('Common','DimC_File',sIniFile));
      end;
      Out_Count := 1;
      application.ProcessMessages;
   end;

  while (GEN < GNumber) and (not pbExit) do
  begin
   i := Random(10000);
   u := Random(2);
   if u = 1 then
      RandSeed := strtoint(FormatDateTime('hhnnss',now))-i
   else
      RandSeed := strtoint(FormatDateTime('hhnnss',now))+i;

   try
      DumpData;
      Wait(Gen);
      if pbExit then
         Goto Exit_Point;

      reset(Data) ;
      for i := 0 to XAmount do
        read(Data,X[i]);
      readln(Data);
      for i := 0 to XAmount do
         read(Data,Y[i]);
      CloseFile(Data);
      if piExVar_No > 0 then
      begin
         for i := 1 to piExVar_No do
            read(ExData,ExVar[i]);
         readln(ExData);
         if eof(ExData) then
         begin
            closefile(ExData);
            Reset(ExData);
         end;
      end;
      if piDoDim = 0 then
      begin
         reset(DimAFile);
         reset(DimBFile);
         reset(DimCFile);
         for i := 1 to WinSize + 11 do
         begin
            readln(DimAFile,DimA[i]);
            readln(DimBFile,DimB[i]);
            readln(DimCFile,DimC[i]);
         end;
         closefile(DimAFile);
         closefile(DimBFile);
         closefile(DimCFile);
      end;
//------------------------------------------------------------------------
     Gen := Gen + 1;
     OutPut_msg('Processing Generation     '+ inttostr(Gen));
     application.ProcessMessages;
     First := Abs(Individual-First);
     Second := Abs(Individual-Second);

     if ((Gen mod G_Gap)=1) or (G_Gap=1) then
     begin
        for i := 1 to Individual do
        begin
           Value := Random(1000) / 1000;
           begin
              iTry := 0;
              repeat
                 Value := Random(Individual);
                 if Value <= Reproduct then
                    Reproduction(i)
                 else
                    if Value <=Reproduct+Mutation then
                    begin
                       if Mutmode = 0 then
                          Mutate0(i)
                       else
                          Mutate1(i);
                    end
                    else
                       Crossover(i);

                 ReadTree(FileIndex_L[Second+i]);
                 ESum1 := 0.0;
                 ESum2 := 0.0;
                 for u := 111 downto 102 do
                 begin
                   Vij2 := ComputeVij(u);
                   ESum1 := ESum1 + Abs((Vij2-DimA[u])/DimA[u]);
                 end;
                 FreeQueue(Queue);

                 ReadTree(FileIndex_L[First+i]);
                 ESum2 := 0.0;
                 for u := 111 downto 102 do
                 begin
                    Vij2 := ComputeVij(u);
                    ESum2 := ESum2 + Abs((Vij2-DimA[u])/DimA[u]);
                 end;
                 FreeQueue(Queue);
                 iTry := iTry + 1;
               until (ESum1 < ESum2) or (iTry >= TryNo);
               if ESum1 < ESum2 then
               begin
                  ReadTree(FileIndex_L[Second+i]);
                  Mape[i] := ComputeMape;
                  FreeQueue(Queue);
               end
               else
               begin
                  ReadTree(FileIndex_L[First+i]);
                  Mape[i] := ComputeMape;
                  WriteTree(Second+i);
                  FreeQueue(Queue);
               end;
           end;
        end;
     end
     else
     begin
        for i := 1 to Individual do
        begin
           ReadTree(FileIndex_L[First+i]);
           Mape[i] := ComputeMape;
           WriteTree(Second+i);
           FreeQueue(Queue);
        end
     end;

     if FileExists(psWaitPath+'Mape.txt') then
        repeat
            application.ProcessMessages;
        until DeleteFile(psWaitPath+'Mape.txt');

     assignfile(MapeFile,psWaitPath+'Mape.txt');
     rewrite(MapeFile);
     for i := 1 to Individual do
        writeln(MapeFile,Mape[i]);
     closefile(MapeFile);

     if fileexists(psWaitPath+'Gen' + inttostr(Gen-1)) then
       repeat
           application.ProcessMessages;
       until deletefile(psWaitPath+'Gen' + inttostr(Gen-1));
     application.ProcessMessages;
Exit_Point:

  except
     pbExit := true;
     Myini := TIniFile.Create(ApPath + 'Ver2.ini');
     Myini.WriteString('Common','Exit','1');
     Myini.Free;
  end;
end;      (* end of while *)

   FreeQueue(Queue);
   if not pbExit then
   begin
     Myini := TIniFile.Create(sInifile);
     Myini.WriteString('Common','Stop_No','0');
     Myini.Free;
     CloseFile(Data);
   end
   else
      DumpData;
   Myini := TIniFile.Create(sInifile);
   Myini.WriteString('Common','Running','0');
   Myini.Free;
   application.Terminate;
   exit;
end;

end.
